# Podflow/download_and_build.py
# coding: utf-8

import threading
from Podflow.youtube.build import get_youtube_introduction
from Podflow.message.create_main_rss import create_main_rss
from Podflow.download.youtube_and_bilibili_download import youtube_and_bilibili_download


def get_and_duild():
    get_youtube_introduction()
    create_main_rss()


# 下载并构建YouTube和哔哩哔哩视频模块
def download_and_build():
    thread_download = threading.Thread(target=youtube_and_bilibili_download)
    thread_build = threading.Thread(target=get_and_duild)

    thread_download.start()
    thread_build.start()

    thread_download.join()
    thread_build.join()



# Podflow/download/convert_bytes.py
# coding: utf-8


# 格式化字节模块
def convert_bytes(byte_size, units=None, outweigh=1024):
    if units is None:
        units = [" B", "KB", "MB", "GB"]
    if byte_size is None:
        byte_size = 0
    # 初始单位是字节
    unit_index = 0
    # 将字节大小除以1024直到小于1024为止
    while byte_size > outweigh and unit_index < len(units) - 1:
        byte_size /= 1024.0
        unit_index += 1
    # 格式化结果并返回
    return f"{byte_size:.2f}{units[unit_index]}"



# Podflow/message/xml_original_item.py
# coding: utf-8

import re
import html
import hashlib
from Podflow import gVar
from Podflow.message.title_correction import title_correction


# 生成原有的item模块
def xml_original_item(original_item, channelid_title, change_judgment=False, title_change=None):
    if title_change:
        title_change = []
    def description_change(text, sep, title, channelid_title):
        channelid_title = html.escape(channelid_title)
        if sep in text:
            text_one, text_two = text.split(sep, 1)
            if text_one[0] == "『" and text_one[-1] == "』":
                text = text_two
        elif text:
            if text[0] == "『" and text[-1] == "』":
                text = ""
        if channelid_title not in title:
            if text == "":
                text = f"『{channelid_title}』{text}"
            else:
                text = f"『{channelid_title}』{sep}{text}".replace('\x00', '')
        return text
    guid = re.search(r"(?<=<guid>).+(?=</guid>)", original_item).group()
    title = re.search(r"(?<=<title>).+(?=</title>)", original_item).group()
    if title_change:
        title = title_correction(title, guid, title_change)
    title = title.replace('\x00', '')
    link = re.search(r"(?<=<link>).+(?=</link>)", original_item).group()
    description = re.search(r"(?<=<description>).+(?=</description>)", original_item)
    description = description.group() if description else ""
    if change_judgment:
        description = description_change(description, "&#xA;", title, channelid_title)
    pubDate = re.search(r"(?<=<pubDate>).+(?=</pubDate>)", original_item).group()
    url = re.search(r"(?<=<enclosure url\=\").+?(?=\")", original_item).group()
    url = re.search(r"(?<=/channel_audiovisual/).+/.+\.(m4a|mp4)", url).group()
    if gVar.config["token"]:
        input_string = f"{gVar.config['token']}/channel_audiovisual/{url}"
    else:
        input_string = f"channel_audiovisual/{url}"
    sha256_hash = hashlib.sha256(input_string.encode()).hexdigest()
    url = f"{gVar.config['address']}/channel_audiovisual/{url}?token={sha256_hash}"
    length = re.search(r"(?<=length\=\")[0-9]+(?=\")", original_item).group()
    type_video = re.search(
        r"(?<=type\=\")(video/mp4|audio/x-m4a|audio/mpeg)(?=\")", original_item
    ).group()
    if type_video == "audio/mpeg":
        type_video = "audio/x-m4a"
    itunes_summary = re.search(
        r"(?<=<itunes:summary><\!\[CDATA\[).+(?=\]\]></itunes:summary>)",
        original_item,
        flags=re.DOTALL,
    )
    itunes_summary = itunes_summary.group() if itunes_summary else ""
    if change_judgment:
        itunes_summary = description_change(itunes_summary, "\n", title, channelid_title)
    itunes_image = re.search(
        r"(?<=<itunes:image href\=\").+(?=\"></itunes:image>)", original_item
    )
    itunes_image = itunes_image.group() if itunes_image else ""
    itunes_duration = re.search(
        r"(?<=<itunes:duration>).+(?=</itunes:duration>)", original_item
    ).group()
    itunes_explicit = re.search(
        r"(?<=<itunes:explicit>).+(?=</itunes:explicit>)", original_item
    ).group()
    itunes_order = re.search(
        r"(?<=<itunes:order>).+(?=</itunes:order>)", original_item
    ).group()
    return f"""
        <item>
            <guid>{guid}</guid>
            <title>{title}</title>
            <link>{link}</link>
            <description>{description}</description>
            <pubDate>{pubDate}</pubDate>
            <enclosure url="{url}" length="{length}" type="{type_video}"></enclosure>
            <itunes:author>{title}</itunes:author>
            <itunes:subtitle>{title}</itunes:subtitle>
            <itunes:summary><![CDATA[{itunes_summary}]]></itunes:summary>
            <itunes:image href="{itunes_image}"></itunes:image>
            <itunes:duration>{itunes_duration}</itunes:duration>
            <itunes:explicit>{itunes_explicit}</itunes:explicit>
            <itunes:order>{itunes_order}</itunes:order>
        </item>
"""



# Podflow/message/get_youtube_and_bilibili_video_format.py
# coding: utf-8

from Podflow import gVar
from Podflow.basic.write_log import write_log
from Podflow.message.media_format import media_format


def one_format(id_update_format, id_num):
    entry_id_update_format = id_update_format[0]
    gVar.video_id_update_format[id_num]["url"] = entry_id_update_format["url"]
    gVar.video_id_update_format[id_num]["format"] = entry_id_update_format["duration_and_id"]
    gVar.video_id_update_format[id_num]["title"] = entry_id_update_format["title"]
    gVar.video_id_update_format[id_num]["timestamp"] = entry_id_update_format["timestamp"]
    gVar.video_id_update_format[id_num]["description"] = entry_id_update_format["description"]
    gVar.video_id_update_format[id_num]["main"] = id_num
    gVar.video_id_update_format[id_num]["image"] = entry_id_update_format["image"]
    gVar.video_id_update_format[id_num]["download"] = entry_id_update_format["download"]


# YouTube&哔哩哔哩视频信息模块
def get_youtube_and_bilibili_video_format(id_num, stop_flag, video_format_lock, prepare_animation):
    id_update_format = media_format(
        gVar.video_id_update_format[id_num]["url"],
        id_num,
        gVar.video_id_update_format[id_num]["media"],
        gVar.video_id_update_format[id_num]["quality"],
        gVar.video_id_update_format[id_num]["cookie"],
    )
    for fail_info in ["年龄限制", "需登录", "请求拒绝"]:
        if fail_info in id_update_format:
            if gVar.youtube_cookie:
                gVar.video_id_update_format[id_num]["cookie"] = "channel_data/yt_dlp_youtube.txt"
                id_update_format = media_format(
                    gVar.video_id_update_format[id_num]["url"],
                    id_num,
                    gVar.video_id_update_format[id_num]["media"],
                    gVar.video_id_update_format[id_num]["quality"],
                    gVar.video_id_update_format[id_num]["cookie"],
                )
                if fail_info in id_update_format:
                    id_update_format = f"\x1b[31m{fail_info}\x1b[0m(Cookies错误)"
            else:
                id_update_format = f"\x1b[31m{fail_info}\x1b[0m(需要Cookies)"
            break
    if gVar.video_id_update_format[id_num]["power"] is True and (
            "试看" in id_update_format
            or id_update_format == "无法获取音频ID"
        ):
        id_update_format = "\x1b[31m充电专属\x1b[0m"
    if isinstance(id_update_format, list):
        if len(id_update_format) == 1:
            one_format(id_update_format, id_num)
        else:
            entrys_id = []
            for entry_id_update_format in id_update_format:
                entry_id = entry_id_update_format["id"]
                entrys_id.append(entry_id)
                gVar.video_id_update_format[entry_id] = {
                    "id": gVar.video_id_update_format[id_num]["id"],
                    "media": gVar.video_id_update_format[id_num]["media"],
                    "quality": gVar.video_id_update_format[id_num]["quality"],
                    "url": entry_id_update_format["url"],
                    "name": gVar.video_id_update_format[id_num]["name"],
                    "cookie": gVar.video_id_update_format[id_num]["cookie"],
                    "format": entry_id_update_format["duration_and_id"],
                    "title": entry_id_update_format["title"],
                    "timestamp": entry_id_update_format["timestamp"],
                    "description": entry_id_update_format["description"],
                    "main": id_num,
                    "image": entry_id_update_format["image"],
                    "download": entry_id_update_format["download"],
                    "backward_update": gVar.video_id_update_format[id_num]["backward_update"],
                }
            gVar.video_id_update_format[id_num] = entrys_id
    else:
        with video_format_lock:
            stop_flag[0] = "error"
            prepare_animation.join()
            gVar.video_id_failed.append(id_num)
            write_log(
                f"{gVar.video_id_update_format[id_num]['name']}|{id_num}|{id_update_format}",
                None,
                True,
                False,
            )
            del gVar.video_id_update_format[id_num]



# Podflow/message/title_correction.py
# coding: utf-8

import re


# 标题文本修改
def title_correction(title, video_url, title_changes: list):
    for title_change in title_changes:
        match = title_change.get("match", None)
        mode = title_change["mode"]
        text = title_change["text"]
        table = title_change.get("table", [])

        def add_title(mode, text, title):
            if mode == "add-left":
                return text + title
            elif mode == "add-right":
                return title + text

        if text and text in title:
            return title
        elif video_url in table:
            return add_title(mode, text, title)
        elif match is not None and re.search(match, title):
            if mode == "replace":
                return re.sub(match, text, title)
            else:
                return add_title(mode, text, title)
    return title



# Podflow/message/media_format.py
# coding: utf-8

import yt_dlp
from Podflow.message.fail_message_initialize import fail_message_initialize


class MyLogger:
    def debug(self, msg):
        pass

    def warning(self, msg):
        pass

    def info(self, msg):
        pass

    def error(self, msg):
        pass


def duration_and_formats(video_website, video_url, cookies):
    fail_message, infos = None, []
    try:
        # 初始化 yt_dlp 实例, 并忽略错误
        ydl_opts = {
            "no_warnings": True,
            "quiet": True,  # 禁止非错误信息的输出
            "logger": MyLogger(),
        }
        if cookies:
            if "www.bilibili.com" in video_website:
                ydl_opts["http_headers"] = {
                    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36",
                    "Referer": "https://www.bilibili.com/",
                }
            elif "www.youtube.com" in video_website:
                ydl_opts["http_headers"] = {
                    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36",
                    "Referer": "https://www.youtube.com/",
                }
            ydl_opts["cookiefile"] = cookies  # cookies 是你的 cookies 文件名
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            # 使用提供的 URL 提取视频信息
            if info_dict := ydl.extract_info(f"{video_website}", download=False):
                # 获取视频时长并返回
                entries = info_dict.get("entries", None)
                download_url = info_dict.get("original_url", None)
                if entries:
                    infos.extend(
                        {
                            "title": entry.get("title"),
                            "duration": entry.get("duration"),
                            "formats": entry.get("formats"),
                            "timestamp": entry.get("timestamp"),
                            "id": entry.get("id"),
                            "description": entry.get("description"),
                            "url": entry.get("webpage_url"),
                            "image": entry.get("thumbnail"),
                            "download": {
                                "url": download_url,
                                "num": playlist_num + 1,
                            },
                            "format_note": entry.get("format_note"),
                        }
                        for playlist_num, entry in enumerate(entries)
                    )
                else:
                    infos.append(
                        {
                            "title": info_dict.get("title"),
                            "duration": info_dict.get("duration"),
                            "formats": info_dict.get("formats"),
                            "timestamp": info_dict.get("timestamp"),
                            "id": info_dict.get("id"),
                            "description": info_dict.get("description"),
                            "url": info_dict.get("webpage_url"),
                            "image": info_dict.get("thumbnail"),
                            "download": {"url": download_url, "num": None},
                            "format_note": info_dict.get("format_note"),
                        }
                    )
    except Exception as message_error:
        fail_message = fail_message_initialize(message_error, video_url)
    return fail_message, infos


# 定义条件判断函数
def check_resolution(item, quality):
    if "aspect_ratio" in item and (isinstance(item["aspect_ratio"], (float, int))):
        if item["aspect_ratio"] >= 1:
            return item["height"] <= int(quality)
        else:
            return item["width"] <= int(quality)
    else:
        return False


def check_ext(item, media):
    return item["ext"] == media if "ext" in item else False


def check_vcodec(item):
    if "vcodec" in item:
        return (
            "vp" not in item["vcodec"].lower()
            and "av01" not in item["vcodec"].lower()
            and "hev1" not in item["vcodec"].lower()
        )
    else:
        return False


# 获取最好质量媒体的id
def best_format_id(formats):
    tbr_max = 0.0
    format_id_best = ""
    vcodec_best = ""
    for form in formats:
        if (
            "tbr" in form
            and "drc" not in form["format_id"]
            and form["protocol"] == "https"
            and (isinstance(form["tbr"], (float, int)))
            and form["tbr"] >= tbr_max
        ):
            tbr_max = form["tbr"]
            format_id_best = form["format_id"]
            vcodec_best = form["vcodec"]
    return format_id_best, vcodec_best


# 获取媒体时长和ID模块
def media_format(video_website, video_url, media="m4a", quality="480", cookies=None):
    fail_message = None
    video_id_count, change_error, fail_message, infos = 0, None, "", []
    while (
        video_id_count < 3
        and change_error is None
        and (fail_message is not None or not infos)
    ):
        video_id_count += 1
        fail_message, infos = duration_and_formats(video_website, video_url, cookies)
    if fail_message is not None:
        return fail_message
    lists = []
    for entry in infos:
        duration = entry["duration"]
        formats = entry["formats"]
        if duration == "" or duration is None:
            return "无法获取时长"
        if formats == "" or formats is None:
            return "无法获取格式"
        # 进行筛选
        formats_m4a = list(
            filter(lambda item: check_ext(item, "m4a") and check_vcodec(item), formats)
        )
        (best_formats_m4a, vcodec_best) = best_format_id(formats_m4a)
        if best_formats_m4a == "" or best_formats_m4a is None:
            return (
                "\033[31m试看\033[0m"
                if entry["format_note"] == "试看"
                else "无法获取音频ID"
            )
        duration_and_id = [duration, best_formats_m4a]
        if media == "mp4":
            formats_mp4 = list(
                filter(
                    lambda item: check_resolution(item, quality)
                    and check_ext(item, "mp4")
                    and check_vcodec(item),
                    formats,
                )
            )
            (best_formats_mp4, vcodec_best) = best_format_id(formats_mp4)
            if best_formats_mp4 == "" or best_formats_mp4 is None:
                return (
                    "\033[31m试看\033[0m"
                    if entry["format_note"] == "试看"
                    else "无法获取视频ID"
                )
            duration_and_id.extend((best_formats_mp4, vcodec_best))
        lists.append(
            {
                "duration_and_id": duration_and_id,
                "title": entry.get("title"),
                "timestamp": entry.get("timestamp"),
                "id": entry.get("id"),
                "description": entry.get("description"),
                "url": entry.get("url"),
                "image": entry.get("image"),
                "download": entry.get("download"),
            }
        )
    return lists



# Podflow/message/want_retry.py
# coding: utf-8

import re
import os
from Podflow.basic.write_log import write_log


# 输出需要更新的信息模块
def update_information_display(
    channelid_ids_update, content_id_update, content_id_backward_update, name
):
    if not channelid_ids_update:
        return
    print_channelid_ids_update = f"需更新的{name}频道:\n"
    # 获取命令行字节宽度
    try:
        terminal_width = os.get_terminal_size().columns
    except OSError:
        terminal_width = 47
    # 尝试拆分输出
    try:
        for channelid_key, channelid_value in channelid_ids_update.items():
            if len(print_channelid_ids_update) != len(name) + 8:
                if (
                    len(
                        re.sub(
                            r"\033\[[0-9;]+m",
                            "",
                            print_channelid_ids_update.split("\n")[-1],
                        ).encode("GBK")
                    )
                    + len(f" | {channelid_value}".encode("utf-8"))
                    <= terminal_width
                ):
                    print_channelid_ids_update += " | "
                else:
                    print_channelid_ids_update += "\n"
            if (
                channelid_key in content_id_update
                and channelid_key in content_id_backward_update
            ):
                print_channelid_ids_update += f"\033[34m{channelid_value}\033[0m"
            elif channelid_key in content_id_update:
                print_channelid_ids_update += f"\033[32m{channelid_value}\033[0m"
            elif channelid_key in content_id_backward_update:
                print_channelid_ids_update += f"\033[36m{channelid_value}\033[0m"
            else:
                print_channelid_ids_update += f"\033[33m{channelid_value}\033[0m"
    # 如果含有特殊字符将使用此输出
    except Exception:
        len_channelid_ids_update = len(channelid_ids_update)
        count_channelid_ids_update = 1
        for channelid_key, channelid_value in channelid_ids_update.items():
            if (
                channelid_key in content_id_update
                and channelid_key in content_id_backward_update
            ):
                print_channelid_ids_update += f"\033[34m{channelid_value}\033[0m"
            elif channelid_key in content_id_update:
                print_channelid_ids_update += f"\033[32m{channelid_value}\033[0m"
            elif channelid_key in content_id_backward_update:
                print_channelid_ids_update += f"\033[36m{channelid_value}\033[0m"
            else:
                print_channelid_ids_update += f"\033[33m{channelid_value}\033[0m"
            if count_channelid_ids_update != len_channelid_ids_update:
                if count_channelid_ids_update % 2 != 0:
                    print_channelid_ids_update += " | "
                else:
                    print_channelid_ids_update += "\n"
                count_channelid_ids_update += 1
    write_log(print_channelid_ids_update)



# Podflow/basic/qr_code.py
# coding: utf-8

import math
import qrcode


# 网址二维码模块
def qr_code(data):
    # 创建一个QRCode对象
    qr = qrcode.QRCode(
        version=1,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=1,
        border=0,
    )
    # 设置二维码的数据
    qr.add_data(data)
    # 获取QR Code矩阵
    qr.make(fit=True)
    matrix = qr.make_image(fill_color="black", back_color="white").modules
    # 获取图像的宽度和高度
    width, height = len(matrix), len(matrix)
    height_double = math.ceil(height / 2)
    # 转换图像为ASCII字符
    fonts = ["▀", "▄", "█", " "]
    ascii_art = ""
    for y in range(height_double):
        if (y + 1) * 2 - 1 >= height:
            for x in range(width):
                ascii_art += (
                    fonts[0] if matrix[(y + 1) * 2 - 2][x] is True else fonts[3]
                )
        else:
            for x in range(width):
                if (
                    matrix[(y + 1) * 2 - 2][x] is True
                    and matrix[(y + 1) * 2 - 1][x] is True
                ):
                    ascii_art += fonts[2]
                elif (
                    matrix[(y + 1) * 2 - 2][x] is True
                    and matrix[(y + 1) * 2 - 1][x] is False
                ):
                    ascii_art += fonts[0]
                elif (
                    matrix[(y + 1) * 2 - 2][x] is False
                    and matrix[(y + 1) * 2 - 1][x] is True
                ):
                    ascii_art += fonts[1]
                else:
                    ascii_art += fonts[3]
            ascii_art += "\n"
    print(ascii_art)
    return height_double



# Podflow/meaasge/get_original_rss.py
# coding: utf-8

import re
import hashlib
from Podflow import gVar


# 生成哈希值模块
def create_hash(data):
    data_str = str(data)
    hash_object = hashlib.sha256()
    hash_object.update(data_str.encode())
    return hash_object.hexdigest()


# rss生成哈希值模块
def rss_create_hash(data):
    patterns = [
        r"<lastBuildDate>(\w+), (\d{2}) (\w+) (\d{4}) (\d{2}):(\d{2}):(\d{2}) \+\d{4}</lastBuildDate>",
        r"Podflow_light\.png",
        r"Podflow_dark\.png",
    ]
    replacement = ""
    for pattern in patterns:
        data = re.sub(pattern, replacement, data)
    return create_hash(data)


# 获取原始xml模块
def get_original_rss():
    xmls_original_fail = []
    # 获取原始总xml文件
    try:
        with open(
            f"{gVar.config['filename']}.xml", "r", encoding="utf-8"
        ) as file:  # 打开文件进行读取
            rss_original = file.read()  # 读取文件内容
            get_xmls_original = {
                rss_original_channel: rss_original.split(
                    f"<!-- {{{rss_original_channel}}} -->\n"
                )[1]
                for rss_original_channel in list(
                    set(re.findall(r"(?<=<!-- \{).+?(?=\} -->)", rss_original))
                )
            }
    except FileNotFoundError:  # 文件不存在直接更新
        get_xmls_original = {}
        rss_original = ""
    # 如原始xml无对应的原频道items, 将尝试从对应频道的xml中获取
    for channelid_key in (
        gVar.channelid_youtube_ids | gVar.channelid_bilibili_ids
    ).keys():
        if channelid_key not in get_xmls_original.keys():
            try:
                with open(
                    f"channel_rss/{channelid_key}.xml", "r", encoding="utf-8"
                ) as file:  # 打开文件进行读取
                    youtube_rss_original = file.read()  # 读取文件内容
                    get_xmls_original[channelid_key] = youtube_rss_original.split(
                        f"<!-- {{{channelid_key}}} -->\n"
                    )[1]
            except FileNotFoundError:  # 文件不存在直接更新
                xmls_original_fail.append(channelid_key)
    # 生成原始rss的哈希值
    hash_rss_original = rss_create_hash(rss_original)
    return get_xmls_original, hash_rss_original, xmls_original_fail



# Podflow/main_upload.py
# coding: utf-8

import sys
import threading
from Podflow.upload.upload_print import upload_print
from Podflow.upload.linked_server import handle_discovery, usable_port


def main_upload():
    # 服务发现相关配置
    broadcast_port = 37001  # 服务发现用端口
    service_port = 5000  # 实际服务端口

    broadcast_port = usable_port(broadcast_port, 37100)
    service_port = usable_port(service_port, 5050)
    if broadcast_port and service_port:
        discovery_thread = threading.Thread(
            target=handle_discovery,
            args=(broadcast_port, service_port),
        )
        discovery_thread.start()
    else:
        if not broadcast_port:
            upload_print("\033[31m广播端口被占用\033[0m")
        if not service_port:
            upload_print("\033[31m服务端口被占用\033[0m")
        sys.exit(0)



# Podflow/upload/linked_client.py
# coding: utf-8

import time
import socket
from datetime import datetime
from Podflow.upload.upload_print import upload_print


BROADCAST_PORT = 37000
TIMEOUT = 1  # 搜索超时时间（秒）
MAX_BROADCAST_PORT = 37101  # 尝试广播的最大端口


# 发现局域网内的服务器
def discover_server(broadcast_port, timeout):
    servers = []

    # 创建UDP socket
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.settimeout(timeout)

        try:
            # 发送广播请求
            sock.sendto(b"DISCOVER_SERVER_REQUEST", ("<broadcast>", broadcast_port))
        except Exception as e:
            upload_print(f"广播请求发送失败: {e}")
            return servers

        # 等待响应
        start_time = time.time()
        while time.time() - start_time < timeout:
            try:
                data, addr = sock.recvfrom(1024)
                if data.startswith(b"SERVER_INFO|"):
                    try:
                        port = int(data.decode().split("|")[1])
                        servers.append((addr[0], port))
                    except (IndexError, ValueError):
                        upload_print(f"收到来自 {addr} 的服务响应格式不正确")
            except socket.timeout:
                break
            except Exception as e:
                upload_print(f"接收数据出错: {e}")
                break

    return servers


# 自动发现并连接服务器模块
def connect_server():
    upload_print("正在搜索服务器...")

    current_port = BROADCAST_PORT
    servers = []
    time_print = f"{datetime.now().strftime('%H:%M:%S')}|"

    # 在允许的端口范围内尝试发现服务器
    while current_port < MAX_BROADCAST_PORT:
        print(f"\r{time_print}正在尝试广播端口 {current_port}...", end="")
        servers = discover_server(current_port, TIMEOUT)
        if servers:
            print("")
            break
        current_port += 1

    if not servers:
        upload_print("找不到服务器")
        return

    # 选择第一个找到的服务器
    server_ip, server_port = servers[0]
    upload_print(f"正在连接到{server_ip}:{server_port}")



# Podflow/basic/list_merge_tidy.py
# coding: utf-8


# 合并整形列表模块
def list_merge_tidy(list1, list2=None, length=None):
    if list2 is None:
        list2 = []
    final_list = []
    for item in list1 + list2:
        if item:
            item = item[:length]
        if item not in final_list:
            final_list.append(item)
    return final_list



# Podflow/bilibili/build.py
# coding: utf-8

import re
import html
import contextlib
from datetime import datetime, timezone
from Podflow import gVar
from Podflow.message.xml_rss import xml_rss
from Podflow.basic.file_save import file_save
from Podflow.message.xml_item import xml_item
from Podflow.bilibili.get import get_bilibili_cid
from Podflow.message.format_time import format_time
from Podflow.basic.get_file_list import get_file_list
from Podflow.message.xml_original_item import xml_original_item


def get_items_list(
    guid,
    item,
    channelid_title,
    title_change,
    items_counts,
    output_dir,
    items_list,
):
    pubDate = datetime.fromtimestamp(item["created"], timezone.utc).strftime(
        "%Y-%m-%dT%H:%M:%S%z"
    )
    if guid in items_counts:
        guid_parts = []
        guid_edgeinfos = []
        if "cid" in item:
            pass
        elif "part" in item:
            guid_parts = item["part"]
        elif "edgeinfo" in item:
            guid_edgeinfos = item["edgeinfo"]
        elif "error" in item:
            pass  # 需要添加错误处理
        else:
            guid_cid, guid_type, _ = get_bilibili_cid(
                guid, gVar.channelid_bilibili_ids[output_dir]
            )
            if guid_type == "part":
                guid_parts = guid_cid
            elif guid_type == "edgeinfo":
                guid_edgeinfos = guid_cid
        if guid_parts and items_counts[guid] == len(guid_parts):
            for guid_part in guid_parts:
                guid_part_text = f"{item['title']} Part{guid_part['page']:0{len(str(len(guid_parts)))}}"
                if item["title"] != guid_part["part"]:
                    guid_part_text += f" {guid_part['part']}"
                xml_item_text = xml_item(
                    f"{item['bvid']}_p{guid_part['page']}",
                    output_dir,
                    f"https://www.bilibili.com/video/{guid}?p={guid_part['page']}",
                    channelid_title,
                    html.escape(guid_part_text),
                    html.escape(re.sub(r"\n+", "\n", item["description"])),
                    format_time(pubDate),
                    guid_part["first_frame"],
                    title_change,
                )
                items_list.append(f"{xml_item_text}<!-- {output_dir} -->")
        elif guid_edgeinfos and items_counts[guid] == len(guid_edgeinfos):
            cid_edgeinfos = {
                guid_edgeinfo["cid"]: guid_edgeinfo["title"]
                for guid_edgeinfo in guid_edgeinfos
            }
            for guid_edgeinfo in guid_edgeinfos:
                if guid_edgeinfo["options"]:
                    description = (
                        "〖互动视频〗\n"
                        + "\n".join(
                            f"{option}\t✪{cid_edgeinfos[option_cid]}"
                            for option, option_cid in zip(
                                guid_edgeinfo["options"], guid_edgeinfo["options_cid"]
                            )
                        )
                        + "\n------------------------------------------------\n"
                        + item["description"]
                    )
                else:
                    description = (
                        "〖互动视频〗\nTHE END."
                        + "\n------------------------------------------------\n"
                        + item["description"]
                    )
                guid_edgeinfo_text = f"{item['title']} Part{guid_edgeinfo['num']:0{len(str(len(guid_edgeinfos)))}} {guid_edgeinfo['title']}"
                xml_item_text = xml_item(
                    f"{item['bvid']}_{guid_edgeinfo['cid']}",
                    output_dir,
                    f"https://www.bilibili.com/video/{guid}",
                    channelid_title,
                    html.escape(guid_edgeinfo_text),
                    html.escape(re.sub(r"\n+", "\n", description)),
                    format_time(pubDate),
                    guid_edgeinfo["first_frame"],
                    title_change,
                )
                items_list.append(f"{xml_item_text}<!-- {output_dir} -->")
    else:
        xml_item_text = xml_item(
            item["bvid"],
            output_dir,
            f"https://www.bilibili.com/video/{guid}",
            channelid_title,
            html.escape(item["title"]),
            html.escape(re.sub(r"\n+", "\n", item["description"])),
            format_time(pubDate),
            item["pic"],
            title_change,
        )
        items_list.append(f"{xml_item_text}<!-- {output_dir} -->")


# 生成哔哩哔哩对应channel的需更新的items模块
def bilibili_xml_items(output_dir):
    channelid_bilibili_value = gVar.channelid_bilibili[
        gVar.channelid_bilibili_ids[output_dir]
    ]
    content_id, items_counts = get_file_list(
        output_dir, channelid_bilibili_value["media"]
    )
    items_list = [f"<!-- {output_dir} -->"]
    entry_num = 0
    original_judgment = True
    channelid_title = channelid_bilibili_value["title"]
    title_change = channelid_bilibili_value.get("title_change", [])
    output_dir_value = gVar.channelid_bilibili_rss[output_dir]
    # 最新更新
    for guid in output_dir_value["content"]["list"]:
        if guid not in gVar.video_id_failed and guid in content_id:
            item = output_dir_value["content"]["entry"][guid]
            get_items_list(
                guid,
                item,
                channelid_title,
                title_change,
                items_counts,
                output_dir,
                items_list,
            )
            if item["description"] and item["description"][0] == "『":
                original_judgment = False
        entry_num += 1
        if entry_num >= channelid_bilibili_value["update_size"]:
            break
    items_guid = re.findall(r"(?<=<guid>).+?(?=</guid>)", "".join(items_list))
    # 存量接入
    entry_count = channelid_bilibili_value["last_size"] - len(items_guid)
    if gVar.xmls_original and output_dir in gVar.xmls_original and entry_count > 0:
        xml_num = 0
        for xml in gVar.xmls_original[output_dir].split(f"<!-- {output_dir} -->"):
            xml_guid = re.search(r"(?<=<guid>).+(?=</guid>)", xml)
            if (
                xml_guid
                and xml_guid.group() not in items_guid
                and xml_guid.group() not in gVar.video_id_failed
            ):
                items_list.append(
                    f"{xml_original_item(xml, channelid_title, original_judgment, title_change)}<!-- {output_dir} -->"
                )
                xml_num += 1
            if xml_num >= entry_count:
                break
    # 向后更新
    with contextlib.suppress(KeyError):
        backward = output_dir_value["backward"]
        for backward_guid in backward["list"]:
            if (
                backward_guid not in gVar.video_id_failed
                and backward_guid in content_id
            ):
                backward_item = backward["entry"][backward_guid]
                get_items_list(
                    backward_guid,
                    backward_item,
                    channelid_title,
                    title_change,
                    items_counts,
                    output_dir,
                    items_list,
                )
    # 生成对应xml
    description = html.escape(output_dir_value["content"]["sign"])
    icon = output_dir_value["content"]["face"]
    category = gVar.config["category"]
    title = html.escape(output_dir_value["content"]["name"])
    link = f"https://space.bilibili.com/{output_dir}"
    items = "".join(items_list)
    items = f"""<!-- {{{output_dir}}} -->
{items}
<!-- {{{output_dir}}} -->"""
    return {
        "title": title,
        "link": link,
        "description": description,
        "category": category,
        "icon": icon,
        "items": items,
    }



# Podflow/makeup/make_up_file.py
# coding: utf-8

import os
from Podflow import gVar


# 补全缺失媒体文件到字典模块
def make_up_file():
    channelid_youtube_ids = gVar.channelid_youtube_ids
    for output_dir, name in channelid_youtube_ids.items():
        for file_name in gVar.all_youtube_content_ytid[output_dir]:
            if file_name not in os.listdir(f"channel_audiovisual/{output_dir}"):
                main = file_name.split(".")[0]
                media = file_name.split(".")[1]
                video_id_format = {
                    "id": output_dir,
                    "media": media,
                    "url": f"https://www.youtube.com/watch?v={main}",
                    "name": name,
                    "cookie": None,
                    "main": main,
                }
                if media == "mp4":
                    video_quality = gVar.channelid_youtube[name]["quality"]
                else:
                    video_quality = 1080
                video_id_format["quality"] = video_quality
                gVar.make_up_file_format[main] = video_id_format

    channelid_bilibili_ids = gVar.channelid_bilibili_ids
    for output_dir, name in channelid_bilibili_ids.items():
        for file_name in gVar.all_bilibili_content_bvid[output_dir]:
            if file_name not in os.listdir(f"channel_audiovisual/{output_dir}"):
                main = file_name.split(".")[0][:12]
                if main not in gVar.make_up_file_format:
                    media = file_name.split(".")[1]
                    video_id_format = {
                        "id": output_dir,
                        "media": media,
                        "url": f"https://www.bilibili.com/video/{main}",
                        "name": name,
                        "cookie": "channel_data/yt_dlp_bilibili.txt",
                        "main": main,
                    }
                    if media == "mp4":
                        video_quality = gVar.channelid_bilibili[name]["quality"]
                    else:
                        video_quality = 1080
                    video_id_format["quality"] = video_quality
                    gVar.make_up_file_format[main] = video_id_format



# Podflow/upload/linked_server.py
# coding: utf-8

import socket
from Podflow.httpfs.port_judge import port_judge
from Podflow.upload.upload_print import upload_print


def usable_port(port, max_num):
    hostip = "0.0.0.0"
    while port <= max_num:
        if port_judge(hostip, port):
            return port
        else:
            port += 1
    return None


# 处理服务发现请求的UDP服务模块
def handle_discovery(broadcast_port, service_port):
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.bind(("", broadcast_port))

        upload_print("发现服务已启动...")
        while True:
            data, addr = sock.recvfrom(1024)
            if data == b"DISCOVER_SERVER_REQUEST":
                upload_print(f"来自{addr}的发现请求")
                response = f"SERVER_INFO|{service_port}".encode()
                sock.sendto(response, addr)



# Podflow/youtube/login.py
# coding: utf-8

from datetime import datetime
from Podflow.basic.http_client import http_client
from Podflow.basic.write_log import write_log
from Podflow.netscape.get_cookie_dict import get_cookie_dict


def get_youtube_cookie_fail(arg0):
    print(f"{datetime.now().strftime('%H:%M:%S')}{arg0}")
    write_log("YouTube \033[31m获取cookie失败\033[0m")
    return None


# 获取YouTube cookie模块
def get_youtube_cookie(channelid_youtube_ids):
    if not channelid_youtube_ids:
        return
    youtube_cookie = get_cookie_dict("channel_data/yt_dlp_youtube.txt")
    if youtube_cookie is None:
        write_log("YouTube \033[31m获取cookie失败\033[0m")
        return None
    if response := http_client(
        "https://www.youtube.com", "YouTube主页", 10, 4, True, youtube_cookie
    ):
        html_content = response.text
        if '"LOGGED_IN":true' in html_content:
            print(
                f"{datetime.now().strftime('%H:%M:%S')}|YouTube \033[32m获取cookie成功\033[0m"
            )
            return youtube_cookie
        elif '"LOGGED_IN":false' in html_content:
            return get_youtube_cookie_fail("|登陆YouTube失败")
        else:
            return get_youtube_cookie_fail("|登陆YouTube无法判断")
    else:
        write_log("YouTube \033[31m获取cookie失败\033[0m")
        return None



# Podflow/upload/update_upload.py
# coding: utf-8

import re
import time
from email.utils import parsedate_tz, mktime_tz
from Podflow import gVar
from Podflow.basic.file_save import file_save
from Podflow.message.get_media_name import get_media_name


# 更新并保存上传列表模块
def update_upload():
    # 如果没有开启上传功能，则直接返回
    if not gVar.config["upload"]:
        return
    # 初始化一个字典，用于存储每个输出目录的媒体名称
    media_name = {}
    # 初始化一个列表，用于存储需要上传的媒体部分
    main_upload = []
    # 获取原始上传列表
    upload_original = gVar.upload_original
    # 获取所有项目
    all_items = gVar.all_items
    # 遍历所有项目，获取每个输出目录的媒体名称
    for output_dir, items_dict in all_items.items():
        media_name[output_dir] = get_media_name(items_dict["type"], items_dict["items"])
    # 遍历原始上传列表，筛选出需要上传的媒体部分
    for upload_part in upload_original:
        if (
            upload_part["channel_id"] in media_name
            and upload_part["media_id"] in media_name[upload_part["channel_id"]]
        ):
            main_upload.append(upload_part)

    # 获取需要上传的媒体部分的ID
    media_ids = [item["media_id"] for item in main_upload if "media_id" in item]
    # 遍历每个输出目录的媒体部分，筛选出需要上传的媒体部分
    for output_dir, media_parts in media_name.items():
        for part in media_parts:
            if part not in media_ids:
                # 构造正则表达式，用于匹配媒体部分
                pattern = rf"<!-- {output_dir} -->(?:(?!<!-- {output_dir} -->).)+channel_audiovisual/{output_dir}/{part}.+?<!-- {output_dir} -->"
                pubdate_text = ""
                # 在所有项目中匹配媒体部分
                if match := re.search(
                    pattern, all_items[output_dir]["items"], flags=re.DOTALL
                ):
                    date_text = match.group()
                    # 在匹配的媒体部分中提取发布日期
                    pattern = r"(?<=<pubDate>).+(?=</pubDate>)"
                    if match := re.search(pattern, date_text):
                        pubdate_text = match.group()
                # 如果发布日期存在，则转换为时间戳；否则，使用当前时间戳
                pubdate_text = (
                    mktime_tz(parsedate_tz(pubdate_text))
                    if parsedate_tz(pubdate_text)
                    else int(time.time())
                )

                # 将需要上传的媒体部分添加到列表中
                main_upload.append(
                    {
                        "media_id": part,
                        "channel_id": output_dir,
                        "media_time": pubdate_text,
                        "upload": False,
                        "remove": False,
                        "hash": None,
                    }
                )

    # 将需要上传的媒体部分保存到文件中
    file_save(main_upload, "upload.json", "channel_data")  # 保存到文件



# Podflow/message/get_video_format.py
# coding: utf-8

from Podflow import gVar
from Podflow.basic.write_log import write_log
from Podflow.basic.split_dict import split_dict
from Podflow.message.want_retry import want_retry
from Podflow.message.get_video_format_multithread import get_video_format_multithread


def get_youtube_format_front(ytid_content_update, backward_update):
    for ytid_key, ytid_value in ytid_content_update.items():
        channelid_youtube_value = gVar.channelid_youtube[
            gVar.channelid_youtube_ids_update[ytid_key]
        ]
        # 获取对应文件类型
        yt_id_file = channelid_youtube_value["media"]
        yt_id_failed_count = channelid_youtube_value["want_retry_count"]
        # 如果为视频格式获取分辨率
        if yt_id_file == "mp4":
            yt_id_quality = channelid_youtube_value["quality"]
        else:
            yt_id_quality = None
        for yt_id in ytid_value:
            if want_retry(yt_id, yt_id_failed_count):
                yt_id_format = {
                    "id": ytid_key,
                    "media": yt_id_file,
                    "quality": yt_id_quality,
                    "url": f"https://www.youtube.com/watch?v={yt_id}",
                    "name": gVar.channelid_youtube_ids[ytid_key],
                    "cookie": None,  # 特定视频需要
                    "backward_update": backward_update,
                    "power": None,
                }
                gVar.video_id_update_format[yt_id] = yt_id_format
            else:
                gVar.video_id_failed.append(yt_id)
                write_log(
                    f"{gVar.channelid_youtube_ids[ytid_key]}|{yt_id}|跳过更新",
                    None,
                    False,
                )


def get_bilibili_format_front(bvid_content_update, backward_update):
    for bvid_key, bvid_value in bvid_content_update.items():
        channelid_bilibili_value = gVar.channelid_bilibili[
            gVar.channelid_bilibili_ids_update[bvid_key]
        ]
        # 获取对应文件类型
        bv_id_file = channelid_bilibili_value["media"]
        bv_id_failed_count = channelid_bilibili_value["want_retry_count"]
        # 如果为视频格式获取分辨率
        if bv_id_file == "mp4":
            bv_id_quality = channelid_bilibili_value["quality"]
        else:
            bv_id_quality = None
        for bv_id in bvid_value:
            if want_retry(bv_id, bv_id_failed_count):
                if backward_update:
                    power = gVar.channelid_bilibili_rss[bvid_key]["backward"]["entry"][
                        bv_id
                    ].get("power", None)
                else:
                    power = gVar.channelid_bilibili_rss[bvid_key]["content"]["entry"][
                        bv_id
                    ].get("power", None)
                bv_id_format = {
                    "id": bvid_key,
                    "media": bv_id_file,
                    "quality": bv_id_quality,
                    "url": f"https://www.bilibili.com/video/{bv_id}",
                    "name": gVar.channelid_bilibili_ids[bvid_key],
                    "cookie": "channel_data/yt_dlp_bilibili.txt",
                    "backward_update": backward_update,
                    "power": power,
                }
                gVar.video_id_update_format[bv_id] = bv_id_format
            else:
                gVar.video_id_failed.append(bv_id)
                write_log(
                    f"{gVar.channelid_bilibili_ids[bvid_key]}|{bv_id}|跳过更新",
                    None,
                    False,
                )


# 获取YouTube&哔哩哔哩视频格式信息模块
def get_video_format():
    get_youtube_format_front(gVar.youtube_content_ytid_update, False)
    get_bilibili_format_front(gVar.bilibili_content_bvid_update, False)
    get_youtube_format_front(gVar.youtube_content_ytid_backward_update, True)
    get_bilibili_format_front(gVar.bilibili_content_bvid_backward_update, True)
    # 按参数拆分获取量
    if len(gVar.video_id_update_format) != 0:
        video_id_update_format_list = split_dict(
            gVar.video_id_update_format, gVar.config["preparation_per_count"]
        )
        for wait_animation_num, video_id_update_format_item in enumerate(
            video_id_update_format_list, start=1
        ):
            wait_animation_display_info = (
                "媒体视频 "
                if len(video_id_update_format_list) == 1
                else f"媒体视频|No.{str(wait_animation_num).zfill(2)} "
            )
            # 获取视频信息多线程模块
            get_video_format_multithread(
                video_id_update_format_item, wait_animation_display_info
            )



# Podflow/download/wait_animation.py
# coding: utf-8

import time
from datetime import datetime


# 等待动画模块
def wait_animation(stop_flag, wait_animation_display_info):
    animation = "."
    i = 1
    prepare_youtube_print = datetime.now().strftime("%H:%M:%S")
    while True:
        if stop_flag[0] == "keep":
            print(
                f"\r{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation.ljust(5)}\033[0m",
                end="",
            )
        elif stop_flag[0] == "error":
            print(
                f"\r{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation} \033[31m失败:\033[0m"
            )
            break
        elif stop_flag[0] == "end":
            print(
                f"\r{prepare_youtube_print}|{wait_animation_display_info}\033[34m准备中{animation} 已完成\033[0m"
            )
            break
        if i % 5 == 0:
            animation = "."
        else:
            animation += "."
        i += 1
        time.sleep(0.5)



# Podflow/ffmpeg_judge.py
# coding: utf-8

import sys
import subprocess
import shutil
from Podflow.basic.write_log import write_log


def error_ffmpeg_judge(ffmpeg_worry):
    write_log("FFmpeg 未安装, 请安装后重试")
    print(ffmpeg_worry)
    sys.exit(0)


def ffmpeg_judge():
    ffmpeg_worry = """\033[0mFFmpeg安装方法:
    Ubuntu:
    \033[32msudo apt update
    sudo apt install ffmpeg\033[0m
    CentOS:
    \033[32msudo yum update
    sudo yum install ffmpeg\033[0m
    Debian:
    \033[32msudo apt-get update
    sudo apt-get install ffmpeg\033[0m
    Arch Linux、Fedora:
    \033[32msudo pacman -S ffmpeg
    sudo dnf install ffmpeg\033[0m
    检查FFmpeg版本:
    \033[32mffmpeg -version\033[0m"""

    # 使用 shutil.which 检查 ffmpeg 是否安装
    if shutil.which("ffmpeg") is None:
        error_ffmpeg_judge(ffmpeg_worry)

    try:
        # 执行 ffmpeg 命令获取版本信息
        result = subprocess.run(["ffmpeg", "-version"], capture_output=True, text=True, check=True)
        output = result.stdout.lower()
        # 检查输出中是否包含 ffmpeg 版本信息
        if "ffmpeg version" not in output:
            error_ffmpeg_judge(ffmpeg_worry)
    except FileNotFoundError:
        error_ffmpeg_judge(ffmpeg_worry)



# Podflow/makeup/make_up_file_format_mod.py
# coding: utf-8

import threading
from datetime import datetime
from Podflow import gVar
from Podflow.basic.write_log import write_log
from Podflow.message.media_format import media_format


def makeup_format(video_id, makeup_format_lock):
    id_value = gVar.make_up_file_format[video_id]
    makeup_id_format = media_format(
        id_value["url"],
        id_value["main"],
        id_value["media"],
        id_value["quality"],
        id_value["cookie"],
    )
    for fail_info in ["年龄限制", "需登录"]:
        if fail_info in makeup_id_format:
            if gVar.youtube_cookie:
                gVar.make_up_file_format[video_id][
                    "cookie"
                ] = "channel_data/yt_dlp_youtube.txt"
                makeup_id_format = media_format(
                    id_value["url"],
                    id_value["main"],
                    id_value["media"],
                    id_value["quality"],
                    "channel_data/yt_dlp_youtube.txt",
                )
                if fail_info in makeup_id_format:
                    makeup_id_format = f"\x1b[31m{fail_info}\x1b[0m(Cookies错误)"
            else:
                makeup_id_format = f"\x1b[31m{fail_info}\x1b[0m(需要Cookies)"
            break
    if isinstance(makeup_id_format, list):
        if len(makeup_id_format) == 1:
            entry_id_makeup_format = makeup_id_format[0]
            gVar.make_up_file_format[video_id]["format"] = entry_id_makeup_format[
                "duration_and_id"
            ]
            gVar.make_up_file_format[video_id]["download"] = entry_id_makeup_format[
                "download"
            ]
        else:
            entrys_id = []
            for entry_id_makeup_format in makeup_id_format:
                entry_id = entry_id_makeup_format["id"]
                entrys_id.append(entry_id)
                gVar.make_up_file_format[entry_id] = {
                    "id": id_value["id"],
                    "name": id_value["name"],
                    "media": id_value["media"],
                    "quality": id_value["quality"],
                    "url": entry_id_makeup_format["url"],
                    "cookie": id_value["cookie"],
                    "format": entry_id_makeup_format["duration_and_id"],
                    "main": id_value["main"],
                    "download": entry_id_makeup_format["download"],
                }
            del gVar.make_up_file_format[video_id]
    else:
        with makeup_format_lock:
            write_log(f"{id_value['name']}|{video_id}|{makeup_id_format}")
            gVar.make_up_file_format_fail[video_id] = id_value[
                "id"
            ]  # 将无法补全的媒体添加到失败字典中
            del gVar.make_up_file_format[video_id]


# 补全在rss中缺失的媒体格式信息模块
def make_up_file_format_mod():
    # 判断是否补全
    if len(gVar.make_up_file_format) != 0:
        print(
            f"{datetime.now().strftime('%H:%M:%S')}|补全缺失媒体 \033[34m下载准备中...\033[0m"
        )
    # 创建线程锁
    makeup_format_lock = threading.Lock()
    # 创建线程列表
    makeup_format_threads = []
    for video_id in gVar.make_up_file_format:
        thread = threading.Thread(
            target=makeup_format,
            args=(
                video_id,
                makeup_format_lock,
            ),
        )
        makeup_format_threads.append(thread)
        thread.start()
    # 等待所有线程完成
    for thread in makeup_format_threads:
        thread.join()



# Podflow/config/channge_icon.py
# coding: utf-8

from datetime import datetime, timedelta, timezone
from astral import LocationInfo
from astral.sun import sun
from Podflow.basic.http_client import http_client
from Podflow.basic.write_log import write_log
from Podflow import gVar, default_config


# 获取日出日落并判断昼夜模块
def http_day_and_night(latitude, longitude):
    sun_url = "https://api.sunrise-sunset.org/json"
    sun_data = {
        "lat": latitude,
        "lng": longitude,
        "date": "today",
    }
    sunrise_sunset = http_client(sun_url, "获取日出日落", 3, 5, True, None, sun_data)
    if not sunrise_sunset:
        return None
    try:
        time_dict = sunrise_sunset.json()["results"]
        sunrise = time_dict["sunrise"]
        sunset = time_dict["sunset"]
    except KeyError:
        return None
    # 获取当前时间, 并去除时区
    now = datetime.now()
    # 将日出和日落时间转换为datetime对象
    today = now.date()
    sunrise_time = datetime.strptime(sunrise, "%I:%M:%S %p")
    sunrise_time = sunrise_time.replace(
        year=today.year, month=today.month, day=today.day, tzinfo=timezone.utc
    )
    sunset_time = datetime.strptime(sunset, "%I:%M:%S %p")
    sunset_time = sunset_time.replace(
        year=today.year, month=today.month, day=today.day, tzinfo=timezone.utc
    )
    # 转换日出和日落时间为时间戳
    sunrise_now = sunrise_time.timestamp()
    sunset_now = sunset_time.timestamp()
    today = now.timestamp()
    # 计算昨天及明天日出和日落时间戳
    sunrise_yesterday = sunrise_now - 3600 * 24
    sunset_yesterday = sunset_now - 3600 * 24
    sunrise_tommorrow = sunrise_now + 3600 * 24
    sunset_tommorrow = sunset_now + 3600 * 24
    if sunrise_now < sunset_now:
        return (
            "light"
            if (
                sunrise_now < today < sunset_now
                or sunrise_yesterday < today < sunset_yesterday
                or sunrise_tommorrow < today < sunset_tommorrow
            )
            else "dark"
        )
    if (
        sunrise_now > today > sunset_now
        or sunrise_yesterday > today > sunset_yesterday
        or sunrise_tommorrow > today > sunset_tommorrow
    ):
        return "dark"
    else:
        return "light"


# 根据经纬度判断昼夜模块
def judging_day_and_night(latitude, longitude):
    # 创建一个 LocationInfo 对象, 只提供经纬度信息
    location = LocationInfo("", "", "", latitude=latitude, longitude=longitude)
    # 获取当前日期和时间, 并为其添加时区信息
    now = datetime.now(timezone.utc)
    yesterday = now - timedelta(days=1)
    tommorrow = now + timedelta(days=1)

    def sunrise_sunset(time):
        # 创建一个 Sun 对象
        sun_time = sun(location.observer, date=time)
        # 计算日出和日落时间, 以及日落前和日出后的一小时
        sunrise = sun_time["sunrise"]
        sunset = sun_time["sunset"]
        sunrise_minus_one_hour = sunrise  # - timedelta(hours=1)
        sunset_plus_one_hour = sunset  # + timedelta(hours=1)
        return sunrise_minus_one_hour, sunset_plus_one_hour

    sunrise_now, sunset_now = sunrise_sunset(now)
    sunrise_yesterday, sunset_yesterday = sunrise_sunset(yesterday)
    sunrise_tommorrow, sunset_tommorrow = sunrise_sunset(tommorrow)
    if sunrise_now < sunset_now:
        return (
            "light"
            if (
                sunrise_now < now < sunset_now
                or sunrise_yesterday < now < sunset_yesterday
                or sunrise_tommorrow < now < sunset_tommorrow
            )
            else "dark"
        )
    if (
        sunrise_now > now > sunset_now
        or sunrise_yesterday > now > sunset_yesterday
        or sunrise_tommorrow > now > sunset_tommorrow
    ):
        return "dark"
    else:
        return "light"


def ipinfo():
    if response := http_client("https://ipinfo.io/json/", "", 1, 0):
        data = response.json()
        # 提取经度和纬度
        coordinates = data["loc"].split(",")
        return True, coordinates[0], coordinates[1]
    else:
        return False, None, None


def ipapi():
    if response := http_client("http://ip-api.com/json/", "", 1, 0):
        data = response.json()
        # 提取经度和纬度
        return True, data["lat"], data["lon"]
    else:
        return False, None, None


def freegeoip():
    if response := http_client("https://freegeoip.app/json/", "", 1, 0):
        data = response.json()
        # 提取经度和纬度
        return True, data["latitude"], data["longitude"]
    else:
        return False, None, None


# 根据日出日落修改封面(只适用原封面)模块
def channge_icon():
    config = gVar.config
    if config["icon"] != default_config["icon"]:
        return
    label = False
    # 公网获取经纬度
    label, latitude, longitude = ipinfo()
    if label is False:
        write_log("获取经纬度信息重试中...\033[97m1\033[0m")
        label, latitude, longitude = ipapi()
    if label is False:
        write_log("获取经纬度信息重试中...\033[97m2\033[0m")
        label, latitude, longitude = freegeoip()
    if label is False:
        write_log("获取经纬度信息失败")
    if label:
        picture_name = http_day_and_night(latitude, longitude)
        if not picture_name:
            write_log("获取日出日落失败, 将计算昼夜")
            picture_name = judging_day_and_night(latitude, longitude)
        config["icon"] = (
            f"https://raw.githubusercontent.com/gruel-zxz/podflow/main/Podflow_{picture_name}.png"
        )



# Podflow/main.py
# coding: utf-8

from datetime import datetime
from importlib.metadata import version
from Podflow.main_podcast import main_podcast
from Podflow.parse_arguments import parse_arguments


def main():
    # 获取传入的参数
    parse_arguments()
    # 开始运行
    print(
        f"{datetime.now().strftime('%H:%M:%S')}|Podflow|{version('Podflow')}开始运行....."
    )
    main_podcast()


if __name__ == "__main__":
    main()



# Podflow/makeup/del_makeup_format_fail.py
# coding: utf-8

import re
from Podflow import gVar


# 删除无法补全的媒体模块
def del_makeup_format_fail():
    for video_id, id_value in gVar.make_up_file_format_fail.items():
        pattern_video_fail_item = rf"<!-- {id_value} -->(?:(?!<!-- {id_value} -->).)+?<guid>{video_id}</guid>.+?<!-- {id_value} -->"
        replacement_video_fail_item = f"<!-- {id_value} -->"
        gVar.all_items[id_value]["items"] = re.sub(
            pattern_video_fail_item,
            replacement_video_fail_item,
            gVar.all_items[id_value]["items"],
            flags=re.DOTALL,
        )



# Podflow/message/rss_create_hash.py
# coding: utf-8

import re
import hashlib


# 生成哈希值模块
def create_hash(data):
    data_str = str(data)
    hash_object = hashlib.sha256()
    hash_object.update(data_str.encode())
    return hash_object.hexdigest()


# rss生成哈希值模块
def rss_create_hash(data):
    patterns = [
        r"<lastBuildDate>(\w+), (\d{2}) (\w+) (\d{4}) (\d{2}):(\d{2}):(\d{2}) \+\d{4}</lastBuildDate>",
        r"Podflow_light\.png",
        r"Podflow_dark\.png",
    ]
    replacement = ""
    for pattern in patterns:
        data = re.sub(pattern, replacement, data)
    return create_hash(data)



# Podflow/youtube/get.py
# coding: utf-8

import contextlib
import re
import os
import threading
from datetime import datetime
from Podflow import gVar
from Podflow.basic.http_client import http_client
from Podflow.basic.vary_replace import vary_replace
from Podflow.basic.get_html_dict import get_html_dict
from Podflow.basic.list_merge_tidy import list_merge_tidy


# 从YouTube播放列表获取更新模块
def get_youtube_html_playlists(
    youtube_key,  # YouTube 频道的唯一标识
    youtube_value,  # YouTube 账户或其他标识信息
    guids=None,  # 视频 ID 列表（用于比较已有视频）
    direction_forward=True,  # 控制获取方向，默认向前获取新的视频
    update_size=20,  # 更新数量限制，最多获取 20 个新视频
    youtube_content_ytid_original=None,  # 原始 YouTube 视频 ID 列表
):
    idlist = []  # 存储新获取的 YouTube 视频 ID
    item = {}  # 存储视频信息（标题、描述、封面等）
    threads = []  # 线程列表，用于并发获取视频详细信息
    fail = []  # 存储获取失败的视频 ID

    if guids is None:
        guids = [""]
    if youtube_content_ytid_original is None:
        youtube_content_ytid_original = []

    try:
        videoid_start = guids[0] if direction_forward else guids[-1]  # 获取起始视频 ID
    except IndexError:
        videoid_start = ""  # 处理空列表情况，避免 IndexError

    # 获取视频详细信息的内部函数
    def get_video_item(videoid, youtube_value):
        yt_Initial_Player_Response = get_html_dict(
            f"https://www.youtube.com/watch?v={videoid}",
            f"{youtube_value}|{videoid}",
            "ytInitialPlayerResponse",
        )  # 解析 YouTube 页面，获取视频信息
        if not yt_Initial_Player_Response:
            return None  # 若获取失败，则返回 None

        try:
            player_Microformat_Renderer = yt_Initial_Player_Response["microformat"][
                "playerMicroformatRenderer"
            ]
        except (KeyError, TypeError, IndexError, ValueError):
            player_Microformat_Renderer = {}  # 解析失败时，返回空字典
            fail.append(videoid)  # 记录失败的视频 ID

        if player_Microformat_Renderer:
            try:
                item[videoid]["description"] = player_Microformat_Renderer[
                    "description"
                ]["simpleText"]
            except (KeyError, TypeError, IndexError, ValueError):
                item[videoid]["description"] = ""  # 若没有描述，则置为空
            item[videoid]["pubDate"] = player_Microformat_Renderer[
                "publishDate"
            ]  # 获取发布时间
            item[videoid]["image"] = player_Microformat_Renderer["thumbnail"][
                "thumbnails"
            ][0]["url"]  # 获取封面图
            with contextlib.suppress(KeyError, TypeError, IndexError, ValueError):
                fail.remove(videoid)  # 若成功获取，则从失败列表中移除
        else:
            return None  # 若无有效数据，返回 None

    # 获取播放列表数据
    yt_initial_data = get_html_dict(
        f"https://www.youtube.com/watch?v={videoid_start}&list=UU{youtube_key[-22:]}",
        f"{youtube_value} HTML",
        "ytInitialData",
    )  # 解析 YouTube 播放列表页面，获取数据
    if not yt_initial_data:
        return None  # 若获取失败，则返回 None

    try:
        playlists = yt_initial_data["contents"]["twoColumnWatchNextResults"][
            "playlist"
        ]["playlist"]["contents"]
        main_title = yt_initial_data["contents"]["twoColumnWatchNextResults"][
            "playlist"
        ]["playlist"]["ownerName"]["simpleText"]
    except (KeyError, TypeError, IndexError, ValueError):
        return None  # 若解析失败，返回 None

    # 若方向是向前获取（最新视频）或没有起始视频 ID
    if direction_forward or not videoid_start:
        for playlist in playlists:
            videoid = playlist["playlistPanelVideoRenderer"]["videoId"]  # 提取视频 ID
            if (
                playlist["playlistPanelVideoRenderer"]["navigationEndpoint"][
                    "watchEndpoint"
                ]["index"]
                == update_size
            ):
                break  # 如果达到更新上限，则停止
            if videoid not in guids:  # 确保视频 ID 不是已存在的
                title = playlist["playlistPanelVideoRenderer"]["title"][
                    "simpleText"
                ]  # 获取视频标题
                idlist.append(videoid)  # 添加到 ID 列表
                item[videoid] = {"title": title, "yt-dlp": True}  # 记录视频信息
                if videoid in youtube_content_ytid_original:  # 若视频已在原始列表中
                    item[videoid]["yt-dlp"] = False  # 标记为已存在
                    item_thread = threading.Thread(
                        target=get_video_item, args=(videoid, youtube_value)
                    )  # 启动线程获取详细信息
                    item_thread.start()
                    threads.append(item_thread)
    else:  # 处理向后获取（获取较旧的视频）
        reversed_playlists = []
        for playlist in reversed(playlists):
            videoid = playlist["playlistPanelVideoRenderer"]["videoId"]
            if videoid not in guids:
                reversed_playlists.append(playlist)  # 收集未存在的旧视频
            else:
                break  # 如果找到已存在的视频 ID，则停止

        for playlist in reversed(reversed_playlists[-update_size:]):
            videoid = playlist["playlistPanelVideoRenderer"]["videoId"]
            title = playlist["playlistPanelVideoRenderer"]["title"]["simpleText"]
            idlist.append(videoid)
            item[videoid] = {"title": title, "yt-dlp": True}
            if videoid in youtube_content_ytid_original:
                item[videoid]["yt-dlp"] = False
                item_thread = threading.Thread(
                    target=get_video_item, args=(videoid, youtube_value)
                )
                item_thread.start()
                threads.append(item_thread)

    for thread in threads:
        thread.join()  # 等待所有线程完成

    # 处理获取失败的视频
    for videoid in fail:
        get_video_item(videoid, youtube_value)  # 重新尝试获取失败的视频

    if fail:  # 如果仍然有失败的视频
        if direction_forward or not videoid_start:
            for videoid in fail:
                print(
                    f"{datetime.now().strftime('%H:%M:%S')}|{youtube_value}|{videoid} HTML无法更新, 将不获取"
                )
                if videoid in idlist:
                    idlist.remove(videoid)  # 安全地移除视频 ID，避免 `ValueError`
                del item[videoid]  # 删除对应的字典项
        else:
            print(
                f"{datetime.now().strftime('%H:%M:%S')}|{youtube_value} HTML有失败只更新部分"
            )
            index = len(idlist)
            for videoid in fail:
                if videoid in idlist:
                    index = min(idlist.index(videoid), index)  # 计算最早失败视频的索引
            idlist_fail = idlist[index:]  # 截取失败的视频 ID 列表
            idlist = idlist[:index]  # 只保留成功的视频 ID
            for videoid in idlist_fail:
                if videoid in idlist:
                    idlist.remove(videoid)  # 安全删除失败视频 ID

    return {"list": idlist, "item": item, "title": main_title}  # 返回最终结果


def get_youtube_shorts_id(youtube_key, youtube_value):
    videoIds = []
    url = f"https://www.youtube.com/channel/{youtube_key}/shorts"
    if data := get_html_dict(url, youtube_value, "ytInitialData"):
        with contextlib.suppress(KeyError, TypeError, IndexError, ValueError):
            items = data["contents"]["twoColumnBrowseResultsRenderer"]["tabs"][2][
                "tabRenderer"
            ]["content"]["richGridRenderer"]["contents"]
            for item in items:
                videoId = item["richItemRenderer"]["content"]["shortsLockupViewModel"][
                    "onTap"
                ]["innertubeCommand"]["reelWatchEndpoint"]["videoId"]
                videoIds.append(videoId)
    return videoIds


# 更新Youtube频道xml模块
def youtube_rss_update(
    youtube_key,
    youtube_value,
    pattern_youtube_varys,
    pattern_youtube404,
    pattern_youtube_error,
):
    channelid_youtube = gVar.channelid_youtube
    channelid_youtube_rss = gVar.channelid_youtube_rss
    channelid_youtube_ids_update = gVar.channelid_youtube_ids_update
    # 获取已下载媒体名称
    youtube_media = (
        ("m4a", "mp4")  # 根据 channelid_youtube 的媒体类型选择文件格式
        if channelid_youtube[youtube_value]["media"] == "m4a"
        else ("mp4",)  # 如果不是 m4a，则只选择 mp4
    )
    try:
        # 遍历指定目录下的所有文件，筛选出以 youtube_media 结尾的文件
        youtube_content_ytid_original = [
            os.path.splitext(file)[0]  # 获取文件名（不包括扩展名）
            for file in os.listdir(f"channel_audiovisual/{youtube_key}")  # 指定的目录
            if file.endswith(youtube_media)  # 筛选文件
        ]
    except Exception:
        # 如果发生异常，设置为空列表
        youtube_content_ytid_original = []
    try:
        # 获取原始XML中的内容
        original_item = gVar.xmls_original[youtube_key]
        guids = re.findall(r"(?<=<guid>).+(?=</guid>)", original_item)  # 查找所有guid
    except KeyError:
        # 如果没有找到对应的key，则guids为空
        guids = []
    # 构建 URL
    youtube_url = f"https://www.youtube.com/feeds/videos.xml?channel_id={youtube_key}"
    youtube_response = http_client(youtube_url, youtube_value)  # 请求YouTube数据
    youtube_html_playlists = None
    youtube_channel_response = None
    if youtube_response is not None and re.search(
        pattern_youtube404, youtube_response.text, re.DOTALL
    ):
        youtube_url = f"https://www.youtube.com/channel/{youtube_key}"
        youtube_channel_response = http_client(youtube_url, f"{youtube_value} HTML")
        if youtube_channel_response is not None:
            pattern_youtube_error_mark = False
            for pattern_youtube_error_key in pattern_youtube_error:
                if pattern_youtube_error_key in youtube_channel_response.text:
                    pattern_youtube_error_mark = True
                    youtube_response = youtube_channel_response
                    break
            if not pattern_youtube_error_mark:
                # 检查响应是否有效，最多重试3次
                for _ in range(3):
                    if youtube_html_playlists := get_youtube_html_playlists(
                        youtube_key,
                        youtube_value,
                        [
                            elem
                            for elem in guids
                            if elem in youtube_content_ytid_original
                        ],  # 仅选择已下载的guids
                        True,
                        channelid_youtube[youtube_value]["update_size"],
                        youtube_content_ytid_original,
                    ):
                        break
        shorts_ytid = []
    elif youtube_response is not None and channelid_youtube[youtube_value]["NoShorts"]:
        shorts_ytid = get_youtube_shorts_id(youtube_key, youtube_value)
        gVar.video_id_failed += shorts_ytid  # 将Shorts视频添加到失败列表中
    else:
        shorts_ytid = []
    # 读取原Youtube频道xml文件并判断是否要更新
    try:
        with open(
            f"channel_id/{youtube_key}.txt",
            "r",
            encoding="utf-8",  # 以utf-8编码打开文件
        ) as file:
            youtube_content_original = file.read()  # 读取文件内容
            youtube_content_original_clean = vary_replace(
                pattern_youtube_varys, youtube_content_original
            )  # 清洗内容
    except FileNotFoundError:  # 如果文件不存在
        youtube_content_original = None
        youtube_content_original_clean = None
    if youtube_html_playlists is not None:  # 如果有新播放列表
        channelid_youtube_rss[youtube_key] = {
            "content": youtube_html_playlists,
            "type": "dict",
        }
        if youtube_html_playlists["item"]:
            channelid_youtube_ids_update[youtube_key] = youtube_value  # 更新标识
        youtube_content_ytid = youtube_html_playlists["list"]  # 获取视频ID列表
    else:
        if youtube_response is not None:
            # 如果没有新的播放列表，但响应有效
            channelid_youtube_rss[youtube_key] = {
                "content": youtube_response,
                "type": "html",
            }
            youtube_content = youtube_response.text  # 获取响应内容
            if not youtube_channel_response:
                youtube_content_clean = vary_replace(
                    pattern_youtube_varys, youtube_content
                )  # 清洗内容
                if (
                    youtube_content_clean != youtube_content_original_clean
                    and youtube_response
                ):  # 判断是否要更新
                    channelid_youtube_ids_update[youtube_key] = (
                        youtube_value  # 更新标识
                    )
        else:
            # 如果没有响应，使用原始内容
            channelid_youtube_rss[youtube_key] = {
                "content": youtube_content_original,
                "type": "text",
            }
            youtube_content = youtube_content_original
        try:
            # 从内容中提取视频ID
            youtube_content_ytid = re.findall(
                r"(?<=<id>yt:video:).{11}(?=</id>)", youtube_content
            )
        except TypeError:
            youtube_content_ytid = []  # 处理类型错误
        youtube_content_ytid = youtube_content_ytid[
            : channelid_youtube[youtube_value][
                "update_size"
            ]  # 限制视频ID数量
        ]
    youtube_content_new = list_merge_tidy(youtube_content_ytid, guids)  # 合并并去重
    if youtube_content_ytid := [
        exclude
        for exclude in youtube_content_ytid
        if exclude not in youtube_content_ytid_original
        and exclude not in shorts_ytid  # 仅选择新视频ID(并且不是Shorts)
    ]:
        channelid_youtube_ids_update[youtube_key] = youtube_value  # 更新标识
        gVar.youtube_content_ytid_update[youtube_key] = (
            youtube_content_ytid  # 保存更新的视频ID
        )
    # 向后更新
    if channelid_youtube[youtube_value]["BackwardUpdate"] and guids:
        # 计算向后更新的数量
        backward_update_size = channelid_youtube[youtube_value]["last_size"] - len(
            youtube_content_new
        )
        if backward_update_size > 0:
            for _ in range(3):
                # 获取历史播放列表
                if youtube_html_backward_playlists := get_youtube_html_playlists(
                    youtube_key,
                    youtube_value,
                    guids,
                    False,
                    min(
                        backward_update_size,
                        channelid_youtube[youtube_value]["BackwardUpdate_size"],
                    ),
                    youtube_content_ytid_original,
                ):
                    break
            if youtube_html_backward_playlists:
                backward_list = youtube_html_backward_playlists[
                    "list"
                ]  # 获取向后更新的列表
                for guid in backward_list.copy():
                    if guid in youtube_content_new:
                        backward_list.remove(guid)  # 从列表中移除已更新的GUID
            if youtube_html_backward_playlists and backward_list:
                channelid_youtube_ids_update[youtube_key] = youtube_value  # 更新标识
                channelid_youtube_rss[youtube_key].update(
                    {"backward": youtube_html_backward_playlists}
                )  # 添加向后更新内容
                youtube_content_ytid_backward = []
                youtube_content_ytid_backward.extend(
                    guid
                    for guid in backward_list
                    if guid not in youtube_content_ytid_original
                )
                if youtube_content_ytid_backward:
                    gVar.youtube_content_ytid_backward_update[youtube_key] = (
                        youtube_content_ytid_backward  # 保存向后更新的ID
                    )



# Podflow/makeup/make_up_file_mod.py
# coding: utf-8

import os
from Podflow import gVar
from Podflow.basic.write_log import write_log
from Podflow.download.dl_aideo_video import dl_aideo_video


# 下载补全Youtube和哔哩哔哩视频模块
def make_up_file_mod():
    for video_id, id_value in gVar.make_up_file_format.items():
        media = id_value["media"]
        id_num = id_value["id"]
        if f"{video_id}.{media}" not in os.listdir(f"channel_audiovisual/{id_num}"):
            name = id_value["name"]
            write_log(f"{name}|{video_id} 缺失并重新下载")
            if dl_aideo_video(
                video_id,
                id_num,
                media,
                id_value["format"],
                gVar.config["retry_count"],
                id_value["download"]["url"],
                name,
                id_value["cookie"],
                id_value["download"]["num"],
            ):
                gVar.video_id_failed.append(video_id)
                write_log(f"{id_value['name']}|{video_id} \033[31m无法下载\033[0m")



# Podflow/httpfs/port_judge.py
# coding: utf-8

import socket


def port_judge(hostip, port):
    try:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            sock.bind((hostip, port))
            return True
    except OSError:
        return False



# Podflow/config/get_channelid_id.py
# coding: utf-8

from datetime import datetime


# 读取频道ID模块
def get_channelid_id(channelid, idname):
    output_name = ""
    if idname == "youtube":
        output_name = "YouTube"
    elif idname == "bilibili":
        output_name = "BiliBili"
    if channelid:
        channelid_ids = dict(
            {channel["id"]: key for key, channel in channelid.items()}
        )
        print(f"{datetime.now().strftime('%H:%M:%S')}|读取{output_name}频道的channelid成功")
    else:
        channelid_ids = {}
    return channelid_ids



# Podflow/httpfs/app_bottle.py
# coding: utf-8

import os
import hashlib
from datetime import datetime
import cherrypy
from bottle import Bottle, abort, redirect, request, static_file
from Podflow import gVar
from Podflow.basic.write_log import write_log


class bottle_app:
    # Bottle和Cherrypy初始化模块
    def __init__(self):
        self.app_bottle = Bottle()  # 创建 Bottle 应用
        self.bottle_print = []  # 存储打印日志
        self.setup_routes()

    def setup_routes(self):
        self.app_bottle.route("/", callback=self.home)
        self.app_bottle.route("/shutdown", callback=self.shutdown)
        self.app_bottle.route("/favicon.ico", callback=self.favicon)
        self.app_bottle.route("/<filename:path>", callback=self.serve_static)

    # 判断token是否正确的验证模块
    def token_judgment(self, token, VALID_TOKEN="", filename="", foldername=""):
        if foldername != "channel_audiovisual/":
            # 对于其他文件夹, 采用常规的 Token 验证
            return VALID_TOKEN == "" or token == VALID_TOKEN
        if (
            VALID_TOKEN == ""
            and token == hashlib.sha256(f"{filename}".encode()).hexdigest()
        ):  # 如果没有配置 Token, 则使用文件名的哈希值
            return True
        elif (
            token == hashlib.sha256(f"{VALID_TOKEN}/{filename}".encode()).hexdigest()
        ):  # 使用验证 Token 和文件名的哈希值
            return True
        else:
            return False

    # 添加至bottle_print模块
    def add_bottle_print(self, client_ip, filename, status):
        # 后缀
        suffixs = [".mp4", ".m4a", ".xml", ".ico"]
        # 设置状态码对应的颜色
        status_colors = {
            200: "\033[32m",  # 绿色 (成功)
            401: "\033[31m",  # 红色 (未经授权)
            404: "\033[35m",  # 紫色 (未找到)
            303: "\033[33m",  # 黄色 (重定向)
            206: "\033[36m",  # 青色 (部分内容)
        }
        # 默认颜色
        color = status_colors.get(status, "\033[0m")
        status = f"{color}{status}\033[0m"
        now_time = datetime.now().strftime("%H:%M:%S")
        client_ip = f"\033[34m{client_ip}\033[0m"
        if gVar.config["httpfs"]:
            write_log(
                f"{client_ip} {filename} {status}",
                None,
                False,
                True,
                None,
                "httpfs.log",
            )
        for suffix in suffixs:
            filename = filename.replace(suffix, "")
        self.bottle_print.append(f"{now_time}|{client_ip} {filename} {status}")

    # CherryPy 服务器打印模块
    def cherry_print(self, flag_judgment=True):
        if flag_judgment:
            gVar.server_process_print_flag[0] = "keep"
        if (
            gVar.server_process_print_flag[0] == "keep"
            and self.bottle_print
        ):  # 如果设置为保持输出, 则打印日志
            for info_print in self.bottle_print:
                print(info_print)
            self.bottle_print.clear()

    # 主路由处理根路径请求
    def home(self):
        VALID_TOKEN = gVar.config["token"]  # 从配置中读取主验证 Token

        # 输出请求日志的函数
        def print_out(status):
            client_ip = request.remote_addr  # 获取客户端 IP 地址
            client_port = request.environ.get("REMOTE_PORT")  # 获取客户端端口
            if client_port:
                client_ip = f"{client_ip}:{client_port}"  # 如果有端口信息, 则包括端口
            self.add_bottle_print(client_ip, "/", status)  # 添加日志信息
            self.cherry_print(False)

        token = request.query.get("token")  # 获取请求中的 Token
        if self.token_judgment(token, VALID_TOKEN):  # 验证 Token
            print_out(303)  # 如果验证成功, 输出 200 状态
            return redirect("https://github.com/gruel-zxz/podflow")  # 返回正常响应
        else:
            print_out(401)  # 如果验证失败, 输出 401 状态
            abort(401, "Unauthorized: Invalid Token")  # 返回未经授权错误

    # 路由处理关闭服务器的请求
    def shutdown(self):
        Shutdown_VALID_TOKEN = "shutdown"
        Shutdown_VALID_TOKEN += datetime.now().strftime("%Y%m%d%H%M%S")
        Shutdown_VALID_TOKEN += os.urandom(32).hex()
        Shutdown_VALID_TOKEN = hashlib.sha256(
            Shutdown_VALID_TOKEN.encode()
        ).hexdigest()  # 用于服务器关闭的验证 Token

        # 输出关闭请求日志的函数
        def print_out(status):
            client_ip = request.remote_addr
            client_port = request.environ.get("REMOTE_PORT")
            if client_port:
                client_ip = f"{client_ip}:{client_port}"
            self.add_bottle_print(client_ip, "shutdown", status)
            self.cherry_print(False)

        token = request.query.get("token")  # 获取请求中的 Token
        if self.token_judgment(
            token, Shutdown_VALID_TOKEN
        ):  # 验证 Token 是否为关闭用的 Token
            print_out(200)  # 如果验证成功, 输出 200 状态
            cherrypy.engine.exit()  # 使用 CherryPy 提供的停止功能来关闭服务器
            return "Shutting down..."  # 返回关机响应
        else:
            print_out(401)  # 如果验证失败, 输出 401 状态
            abort(401, "Unauthorized: Invalid Token")  # 返回未经授权错误

    # 路由处理 favicon 请求
    def favicon(self):
        client_ip = request.remote_addr
        if client_port := request.environ.get("REMOTE_PORT"):
            client_ip = f"{client_ip}:{client_port}"
        self.add_bottle_print(client_ip, "favicon.ico", 303)  # 输出访问 favicon 的日志
        self.cherry_print(False)
        return redirect(
            "https://raw.githubusercontent.com/gruel-zxz/podflow/main/Podflow.png"
        )  # 重定向到图标 URL

    # 路由处理静态文件请求
    def serve_static(self, filename):
        VALID_TOKEN = gVar.config["token"]  # 从配置中读取主验证 Token
        # 定义要共享的文件路径
        bottle_filename = gVar.config["filename"]  # 从配置中读取文件名
        shared_files = {
            bottle_filename.lower(): f"{bottle_filename}.xml",  # 文件路径映射, 支持大小写不敏感的文件名
            f"{bottle_filename.lower()}.xml": f"{bottle_filename}.xml",  # 同上, 支持带 .xml 后缀
        }
        bottle_channelid = (
            gVar.channelid_youtube_ids_original
            | gVar.channelid_bilibili_ids_original
            | {"channel_audiovisual/": "", "channel_rss/": ""}
        )  # 合并多个频道 ID
        token = request.query.get("token")  # 获取请求中的 Token

        # 输出文件请求日志的函数
        def print_out(filename, status):
            client_ip = request.remote_addr
            client_port = request.environ.get("REMOTE_PORT")
            if client_port:
                client_ip = f"{client_ip}:{client_port}"
            for (
                bottle_channelid_key,
                bottle_channelid_value,
            ) in bottle_channelid.items():
                filename = filename.replace(
                    bottle_channelid_key, bottle_channelid_value
                )  # 替换频道路径
                if status == 200 and request.headers.get(
                    "Range"
                ):  # 如果是部分请求, 则返回 206 状态
                    status = 206
            self.add_bottle_print(client_ip, filename, status)  # 输出日志
            self.cherry_print(False)

        # 文件是否存在检查的函数
        def file_exist(token, VALID_TOKEN, filename, foldername=""):
            if self.token_judgment(
                token, VALID_TOKEN, filename, foldername
            ):  # 验证 Token
                if os.path.exists(filename):  # 如果文件存在, 返回文件
                    print_out(filename, 200)
                    return static_file(filename, root=".")
                else:  # 如果文件不存在, 返回 404 错误
                    print_out(filename, 404)
                    abort(404, "File not found")
            else:  # 如果 Token 验证失败, 返回 401 错误
                print_out(filename, 401)
                abort(401, "Unauthorized: Invalid Token")

        # 处理不同的文件路径
        if filename in ["channel_audiovisual/", "channel_rss/"]:
            print_out(filename, 404)
            abort(404, "File not found")
        elif filename.startswith("channel_audiovisual/"):
            return file_exist(token, VALID_TOKEN, filename, "channel_audiovisual/")
        elif filename.startswith("channel_rss/") and filename.endswith(".xml"):
            return file_exist(token, VALID_TOKEN, filename)
        elif filename.startswith("channel_rss/"):
            return file_exist(token, VALID_TOKEN, f"{filename}.xml")
        elif filename.lower() in shared_files:
            return file_exist(token, VALID_TOKEN, shared_files[filename.lower()])
        else:
            print_out(filename, 404)  # 如果文件路径未匹配, 返回 404 错误
            abort(404, "File not found")


bottle_app_instance = bottle_app()



# Podflow/message/xml_item.py
# coding: utf-8

import os
import html
import hashlib
from Podflow.message.title_correction import title_correction
from Podflow.basic.time_format import time_format
from Podflow.basic.get_duration import get_duration
from Podflow import gVar


# 生成item模块
def xml_item(
    video_url,
    output_dir,
    video_website,
    channelid_title,
    title,
    description,
    pubDate,
    image,
    title_change=None,
):
    if title_change is None:
        title_change = []
    channelid_title = html.escape(channelid_title)
    if title_change:
        title = title_correction(title, video_url, title_change)
    # 查看标题中是否有频道名称如无添加到描述中并去除空字符
    title = title.replace("\x00", "")
    if channelid_title not in title:
        if description == "":
            description = f"『{channelid_title}』{description}"
        else:
            description = f"『{channelid_title}』\n{description}".replace("\x00", "")
    # 更换描述换行符
    replacement_description = description.replace("\n", "&#xA;").replace("\t", "&#x9;")
    # 获取文件后缀和文件字节大小
    if os.path.exists(f"channel_audiovisual/{output_dir}/{video_url}.mp4"):
        video_length_bytes = os.path.getsize(
            f"channel_audiovisual/{output_dir}/{video_url}.mp4"
        )
        output_format = "mp4"
        video_type = "video/mp4"
    else:
        if os.path.exists(f"channel_audiovisual/{output_dir}/{video_url}.m4a"):
            video_length_bytes = os.path.getsize(
                f"channel_audiovisual/{output_dir}/{video_url}.m4a"
            )
        else:
            video_length_bytes = 0
        output_format = "m4a"
        video_type = "audio/x-m4a"
    # 获取文件时长
    duration = time_format(
        get_duration(f"channel_audiovisual/{output_dir}/{video_url}.{output_format}")
    )
    # 生成url
    if gVar.config["token"]:
        input_string = f"{gVar.config['token']}/channel_audiovisual/{output_dir}/{video_url}.{output_format}"
    else:
        input_string = f"channel_audiovisual/{output_dir}/{video_url}.{output_format}"
    sha256_hash = hashlib.sha256(input_string.encode()).hexdigest()
    url = f"{gVar.config['address']}/channel_audiovisual/{output_dir}/{video_url}.{output_format}?token={sha256_hash}"
    # 回显对应的item
    return f"""
        <item>
            <guid>{video_url}</guid>
            <title>{title}</title>
            <link>{video_website}</link>
            <description>{replacement_description}</description>
            <pubDate>{pubDate}</pubDate>
            <enclosure url="{url}" length="{video_length_bytes}" type="{video_type}"></enclosure>
            <itunes:author>{title}</itunes:author>
            <itunes:subtitle>{title}</itunes:subtitle>
            <itunes:summary><![CDATA[{description}]]></itunes:summary>
            <itunes:image href="{image}"></itunes:image>
            <itunes:duration>{duration}</itunes:duration>
            <itunes:explicit>no</itunes:explicit>
            <itunes:order>1</itunes:order>
        </item>
"""



# Podflow/message/format_time.py
# coding: utf-8

import time
from datetime import datetime, timedelta, timezone


# 格式化时间及时区模块
def format_time(time_str):
    original_tz = timezone.utc  # 原始时区为UTC
    # 解析时间字符串并转换为datetime对象
    dt = datetime.fromisoformat(
        time_str[:-6] if time_str[-3] == ":" else time_str[:-5]
    ).replace(tzinfo=original_tz)
    # 转换为目标时区
    if time_str[-3] == ":":
        tz = timedelta(
            hours=int(time_str[-6:-3]), minutes=int(f"{time_str[-6]}{time_str[-2:]}")
        )
    else:
        tz = timedelta(
            hours=int(time_str[-5:-2]), minutes=int(f"{time_str[-5]}{time_str[-2:]}")
        )
    dt -= tz
    target_tz = timezone(timedelta(seconds=-(time.timezone + time.daylight)))
    dt_target = dt.astimezone(target_tz)
    return dt_target.strftime("%a, %d %b %Y %H:%M:%S %z")



# Podflow/message/save_rss.py
# coding: utf-8

from Podflow import gVar
from Podflow.basic.qr_code import qr_code
from Podflow.message.xml_rss import xml_rss
from Podflow.basic.file_save import file_save
from Podflow.basic.write_log import write_log
from Podflow.message.backup_zip_save import backup_zip_save
from Podflow.message.display_qrcode_and_url import display_qrcode_and_url


# 保存rss文件模块
def save_rss():
    # 定义一个空列表，用于存储所有rss的items
    main_items = []
    # 遍历gVar.all_items字典，获取每个rss的输出目录和items_dict
    for output_dir, items_dict in gVar.all_items.items():
        # 获取items_dict中的各个字段
        title = items_dict["title"]
        link = items_dict["link"]
        description = items_dict["description"]
        category = items_dict["category"]
        icon = items_dict["icon"]
        items = items_dict["items"]
        # 调用file_save函数，将rss保存到指定目录
        file_save(
            xml_rss(title, link, description, category, icon, items),
            f"{output_dir}.xml",
            "channel_rss",
        )
        # 获取items_dict中的其他字段
        display_rss_address = items_dict["DisplayRSSaddress"]
        qrcode = items_dict["QRcode"]
        id_name = items_dict["ID_Name"]
        id_type = items_dict["type"]
        # 根据id_type获取对应的ids_update
        if id_type == "youtube":
            ids_update = gVar.channelid_youtube_ids_update
        elif id_type == "bilibili":
            ids_update = gVar.channelid_bilibili_ids_update
        else:
            ids_update = {}
        # 调用display_qrcode_and_url函数，显示rss地址和二维码
        display_qrcode_and_url(
            output_dir,
            display_rss_address,
            qrcode,
            id_name,
            ids_update,
        )
        # 如果items_dict中的InmainRSS字段为True，则将items添加到main_items列表中
        if items_dict["InmainRSS"]:
            main_items.append(items)

    # 生成主rss
    overall_rss = xml_rss(
        gVar.config["title"],
        gVar.config["link"],
        gVar.config["description"],
        gVar.config["category"],
        gVar.config["icon"],
        "\n".join(main_items),
    )

    # 保存主rss
    file_save(overall_rss, f"{gVar.config['filename']}.xml")

    # 获取gVar.config中的地址和文件名
    address = gVar.config["address"]
    filename = gVar.config["filename"]
    # 如果gVar.config中的token字段存在，则将token添加到overall_url中
    if token := gVar.config["token"]:
        overall_url = f"{address}/{filename}.xml?token={token}"
    else:
        overall_url = f"{address}/{filename}.xml"
    # 调用write_log函数，记录总播客已更新
    write_log("总播客已更新", f"地址:\n\033[34m{overall_url}\033[0m")
    # 如果gVar.displayed_QRcode中不包含"main"，则调用qr_code函数，显示总播客的二维码，并将"main"添加到gVar.displayed_QRcode中
    if "main" not in gVar.displayed_QRcode:
        qr_code(overall_url)
        gVar.displayed_QRcode.append("main")

    # 备份主rss
    backup_zip_save(overall_rss)



# Podflow/config/build_original.py
# coding: utf-8

from Podflow import gVar, parse
from Podflow.config.get_config import get_config
from Podflow.basic.folder_build import folder_build
from Podflow.config.get_channelid import get_channelid
from Podflow.config.correct_config import correct_config
from Podflow.config.get_channelid_id import get_channelid_id
from Podflow.config.correct_channelid import correct_channelid


def build_original():
    # 获取配置文件config
    gVar.config = get_config(parse.config)
    # 纠正配置信息config
    correct_config()
    # 从配置文件中获取YouTube的频道
    gVar.channelid_youtube = get_channelid("youtube")
    # 从配置文件中获取哔哩哔哩的频道
    gVar.channelid_bilibili = get_channelid("bilibili")
    # 构建文件夹channel_id
    folder_build("channel_id")
    # 构建文件夹channel_audiovisual
    folder_build("channel_audiovisual")
    # 构建文件夹channel_rss
    folder_build("channel_rss")
    # 构建文件夹channel_data
    folder_build("channel_data")
    # 修正channelid_youtube
    gVar.channelid_youtube = correct_channelid(gVar.channelid_youtube, "youtube")
    # 修正channelid_bilibili
    gVar.channelid_bilibili = correct_channelid(gVar.channelid_bilibili, "bilibili")
    # 读取youtube频道的id
    gVar.channelid_youtube_ids = get_channelid_id(gVar.channelid_youtube, "youtube")
    # 复制youtube频道id用于删除已抛弃的媒体文件夹
    gVar.channelid_youtube_ids_original = gVar.channelid_youtube_ids.copy()
    # 读取bilibili频道的id
    gVar.channelid_bilibili_ids = get_channelid_id(gVar.channelid_bilibili, "bilibili")
    # 复制bilibili频道id用于删除已抛弃的媒体文件夹
    gVar.channelid_bilibili_ids_original = gVar.channelid_bilibili_ids.copy()



# Podflow/message/backup_zip_save.py
# coding: utf-8

import zipfile
from datetime import datetime
from Podflow import gVar
from Podflow.basic.write_log import write_log
from Podflow.message.rss_create_hash import rss_create_hash


# xml备份保存模块
def backup_zip_save(file_content):
    def get_file_name():
        # 获取当前的具体时间
        current_time = datetime.now()
        # 格式化输出, 只保留年月日时分秒
        formatted_time = current_time.strftime("%Y%m%d%H%M%S")
        return f"{formatted_time}.xml"

    # 定义要添加到压缩包中的文件名和内容
    compress_file_name = "Podflow_backup.zip"
    # 生成新rss的哈希值
    hash_overall_rss = rss_create_hash(gVar.overall_rss)
    # 使用哈希值判断新老rss是否一致
    if hash_overall_rss == gVar.hash_rss_original:
        judging_save = True
        write_log("频道无更新内容将不进行备份")
    else:
        judging_save = False
    while not judging_save:
        # 获取要写入压缩包的文件名
        file_name_str = get_file_name()
        # 打开压缩文件, 如果不存在则创建
        with zipfile.ZipFile(compress_file_name, "a") as zipf:
            # 设置压缩级别为最大
            zipf.compression = zipfile.ZIP_LZMA
            zipf.compresslevel = 9
            # 检查文件是否已存在于压缩包中
            if file_name_str not in zipf.namelist():
                # 将文件内容写入压缩包
                zipf.writestr(file_name_str, file_content)
                judging_save = True
            else:
                # 如果文件已存在, 输出提示信息
                print(f"{file_name_str}已存在于压缩包中, 重试中...")



# Podflow/bilibili/login.py
# coding: utf-8

import binascii
import json
import os
import re
import time
from datetime import datetime
import requests
from Podflow.basic.file_save import file_save
from Podflow.basic.http_client import http_client
from Podflow.basic.qr_code import qr_code
from Podflow.basic.time_stamp import time_stamp
from Podflow.basic.write_log import write_log
from Podflow.netscape.bulid_netscape import bulid_netscape

try:
    from Cryptodome.Cipher import PKCS1_OAEP
    from Cryptodome.Hash import SHA256
    from Cryptodome.PublicKey import RSA
except Exception:
    from Crypto.Cipher import PKCS1_OAEP
    from Crypto.Hash import SHA256
    from Crypto.PublicKey import RSA


# 获取最新的img_key和sub_key模块
def getWbiKeys(bilibili_cookie=None):
    bilibili_url = "https://api.bilibili.com/x/web-interface/nav"
    if resp := http_client(
        bilibili_url, "获取最新的img_key和sub_key", 10, 4, True, bilibili_cookie
    ):
        resp.raise_for_status()
        json_content = resp.json()
        img_url: str = json_content["data"]["wbi_img"]["img_url"]
        sub_url: str = json_content["data"]["wbi_img"]["sub_url"]
        img_key = img_url.rsplit("/", 1)[1].split(".")[0]
        sub_key = sub_url.rsplit("/", 1)[1].split(".")[0]
        return img_key, sub_key
    else:
        return "", ""


# 申请哔哩哔哩二维码并获取token和URL模块
def bilibili_request_qr_code():
    # 实际申请二维码的API请求
    response = http_client(
        "https://passport.bilibili.com/x/passport-login/web/qrcode/generate",
        "申请BiliBili二维码",
        3,
        5,
        True,
    )
    data = response.json()
    return data["data"]["qrcode_key"], data["data"]["url"]


# 扫码登录哔哩哔哩并返回状态和cookie模块
def bilibili_scan_login(token):
    # 发送GET请求
    response = http_client(
        "https://passport.bilibili.com/x/passport-login/web/qrcode/poll",
        "",
        1,
        1,
        True,
        None,
        {"qrcode_key": token},
    )
    if not response:
        return None, None, None
    data = response.json()
    return data["data"]["code"], response.cookies, data["data"]["refresh_token"]


# 登陆哔哩哔哩模块
def bilibili_login():
    buvid3_and_bnut = http_client(
        "https://www.bilibili.com", "哔哩哔哩主页", 10, 4, True
    ).cookies.get_dict()
    token, url = bilibili_request_qr_code()
    print(f"{datetime.now().strftime('%H:%M:%S')}|请用BiliBili App扫描登录:")
    upward = qr_code(url)
    login_status_change = ""
    time_print = f"{datetime.now().strftime('%H:%M:%S')}|BiliBili "
    while True:
        status, cookie, refresh_token = bilibili_scan_login(token)
        if status == 86101:
            login_status = "\033[0m未扫描\033[0m"
        elif status == 86038:
            login_status = "\033[31m二维码超时, 请重试\033[0m"
        elif status == 86090:
            login_status = "\033[32m扫描成功\033[0m"
        elif status == 0:
            login_status = "\033[32m登陆成功\033[0m"
        else:
            login_status = "\033[31m错误\033[0m"
        if login_status_change != login_status:
            if login_status == "":
                print(f"{time_print}{login_status}".ljust(42), end="")
            else:
                print(f"\r{time_print}{login_status}".ljust(42), end="")
        login_status_change = login_status
        if status == 86038:
            print("")
            return status, refresh_token, upward
        elif status == 0:
            print("")
            cookie["buvid3"] = buvid3_and_bnut.get("buvid3", "")
            cookie["b_nut"] = buvid3_and_bnut.get("b_nut", "")
            return cookie, refresh_token, upward
        time.sleep(1)


# 保存哔哩哔哩登陆成功后的cookies模块
def save_bilibili_cookies():
    bilibili_cookie, refresh_token, upward = bilibili_login()
    if bilibili_cookie == 86038:
        return {"cookie": None}, upward
    bilibili_cookie = requests.utils.dict_from_cookiejar(bilibili_cookie)
    bilibili_data = {"cookie": bilibili_cookie, "refresh_token": refresh_token}
    bulid_netscape("yt_dlp_bilibili", bilibili_cookie)
    return bilibili_data, upward


# 检查哔哩哔哩是否需要刷新模块
def judgment_bilibili_update(cookies):
    url = "https://passport.bilibili.com/x/passport-login/web/cookie/info"
    response = http_client(url, "BiliBili刷新判断", 3, 5, True, cookies)
    response = response.json()
    if response["code"] == 0:
        return response["code"], response["data"]["refresh"]
    else:
        return response["code"], None


# 生成CorrespondPath模块
def getCorrespondPath(ts, key):
    cipher = PKCS1_OAEP.new(key, SHA256)
    encrypted = cipher.encrypt(f"refresh_{ts}".encode())
    return binascii.b2a_hex(encrypted).decode()


def bilibili_cookie_update_data(
    bilibili_cookie, new_bilibili_cookie, bilibili_data, new_refresh_token
):
    new_bilibili_cookie["buvid3"] = bilibili_cookie["buvid3"]
    new_bilibili_cookie["b_nut"] = bilibili_cookie["b_nut"]
    bilibili_data["cookie"] = new_bilibili_cookie
    bilibili_data["refresh_token"] = new_refresh_token
    bulid_netscape("yt_dlp_bilibili", new_bilibili_cookie)
    return bilibili_data


# 哔哩哔哩cookie刷新模块
def bilibili_cookie_update(bilibili_data):
    bilibili_cookie = bilibili_data["cookie"]
    # 获取refresh_csrf
    key = RSA.importKey("""\
-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDLgd2OAkcGVtoE3ThUREbio0Eg
Uc/prcajMKXvkCKFCWhJYJcLkcM2DKKcSeFpD/j6Boy538YXnR6VhcuUJOhH2x71
nzPjfdTcqMz7djHum0qSZA0AyCBDABUqCrfNgCiJ00Ra7GmRj+YCK1NJEuewlb40
JNrRuoEUXpabUzGB8QIDAQAB
-----END PUBLIC KEY-----""")

    # 获取当前时间戳
    ts = time_stamp()
    # 获取refresh_csrf
    refresh_csrf_response = http_client(
        f"https://www.bilibili.com/correspond/1/{getCorrespondPath(ts, key)}",
        "获取refresh_csrf",
        3,
        5,
        True,
        bilibili_cookie,
    )
    if refresh_csrf_match := re.search(
        r'<div id="1-name">(.+?)</div>', refresh_csrf_response.text
    ):
        refresh_csrf_value = refresh_csrf_match[1]
    else:
        return {"cookie": None}
    # 更新bilibili_cookie
    update_cookie_url = (
        "https://passport.bilibili.com/x/passport-login/web/cookie/refresh"
    )
    update_cookie_data = {
        "csrf": bilibili_cookie["bili_jct"],
        "refresh_csrf": refresh_csrf_value,
        "source": "main_web",
        "refresh_token": bilibili_data["refresh_token"],
    }
    update_cookie_response = http_client(
        update_cookie_url,
        "更新BiliBili_cookie",
        3,
        5,
        True,
        bilibili_cookie,
        update_cookie_data,
        "post",
    )
    if update_cookie_response.json()["code"] != 0:
        return {"cookie": None}
    new_bilibili_cookie = requests.utils.dict_from_cookiejar(
        update_cookie_response.cookies
    )
    new_refresh_token = update_cookie_response.json()["data"]["refresh_token"]
    # 确认更新bilibili_cookie
    confirm_cookie_url = (
        "https://passport.bilibili.com/x/passport-login/web/confirm/refresh"
    )
    confirm_cookie_data = {
        "csrf": new_bilibili_cookie["bili_jct"],
        "refresh_token": bilibili_data["refresh_token"],
    }
    confirm_cookie_response = http_client(
        confirm_cookie_url,
        "确认更新BiliBili_cookie",
        3,
        5,
        True,
        new_bilibili_cookie,
        confirm_cookie_data,
        "post",
    )
    if confirm_cookie_response.json()["code"] == 0:
        return bilibili_cookie_update_data(
            bilibili_cookie,
            new_bilibili_cookie,
            bilibili_data,
            new_refresh_token,
        )
    else:
        return {"cookie": None}


def get_bilibili_data_success(bilibili_data, channelid_bilibili_ids):
    print(
        f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[32m获取cookie成功\033[0m"
    )
    img_key, sub_key = getWbiKeys()
    bilibili_data["img_key"] = img_key
    bilibili_data["sub_key"] = sub_key
    bilibili_data["timestamp"] = time.time()
    file_save(bilibili_data, "bilibili_data.json", "channel_data")
    if not os.path.isfile("channel_data/yt_dlp_bilibili.txt"):
        bulid_netscape("yt_dlp_bilibili", bilibili_data["cookie"])
    return channelid_bilibili_ids, bilibili_data


def get_bilibili_data_state(bilibili_data, channelid_bilibili_ids):
    bilibili_login_code, bilibili_login_refresh_token = judgment_bilibili_update(
        bilibili_data["cookie"]
    )
    upward = 0
    try_num = 0
    while try_num < 2 and (
        bilibili_login_code != 0 or bilibili_login_refresh_token is not False
    ):
        if bilibili_login_code != 0:
            if try_num == 0:
                print(
                    f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[31m未登陆\033[0m"
                )
            else:
                print(
                    f"\033[{upward + 3}F\033[{upward + 3}K{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[31m未登陆, 重试第\033[0m{try_num}\033[31m次\033[0m"
                )
            bilibili_data, upward = save_bilibili_cookies()
            try_num += 1
        else:
            print(
                f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[33m需刷新\033[0m"
            )
            bilibili_data = bilibili_cookie_update(bilibili_data)
            if bilibili_data["cookie"]:
                print(
                    f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[32m刷新成功\033[0m"
                )
            else:
                print(
                    f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[31m刷新失败, 重新登陆\033[0m"
                )
        bilibili_login_code, bilibili_login_refresh_token = judgment_bilibili_update(
            bilibili_data["cookie"]
        )
    if bilibili_login_code == 0 and bilibili_login_refresh_token is False:
        return get_bilibili_data_success(bilibili_data, channelid_bilibili_ids)
    write_log("BiliBili \033[31m获取cookie失败\033[0m")
    return {}, {"cookie": None, "timestamp": 0.0}


# 登陆刷新哔哩哔哩并获取data
def get_bilibili_data(channelid_bilibili_ids):
    if not channelid_bilibili_ids:
        return {}, {"cookie": None, "timestamp": 0.0}
    try:
        with open("channel_data/bilibili_data.json", "r") as file:
            bilibili_data = file.read()
        bilibili_data = json.loads(bilibili_data)
    except Exception:
        bilibili_data = {"cookie": None, "timestamp": 0.0}
    if time.time() - bilibili_data["timestamp"] - 60 * 60 > 0:
        return get_bilibili_data_state(bilibili_data, channelid_bilibili_ids)
    print(
        f"{datetime.now().strftime('%H:%M:%S')}|BiliBili \033[33m获取cookie成功\033[0m"
    )
    if not os.path.isfile("channel_data/yt_dlp_bilibili.txt"):
        bulid_netscape("yt_dlp_bilibili", bilibili_data["cookie"])
    return channelid_bilibili_ids, bilibili_data



# Podflow/parse_arguments.py
# coding: utf-8

import argparse
from Podflow import parse


def positive_int(value):
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(f"{value} is not a positive integer")
    return ivalue


# 获取命令行参数并判断
def parse_arguments():
    # 创建 ArgumentParser 对象
    parser = argparse.ArgumentParser(
        description="you can try: Podflow -n 24 -d 3600"
    )
    # 参数
    parser.add_argument(
        "-n",
        "--times",
        nargs=1,
        type=positive_int,
        metavar="NUM",
        help="number of times",
    )
    parser.add_argument(
        "-d",
        "--delay",
        type=positive_int,
        default=1500,
        metavar="NUM",
        help="delay in seconds(default: 1500)",
    )
    parser.add_argument(
        "-c",
        "--config",
        type=str,
        default="config.json",
        metavar="FILE_PATH",
        help="path to the config.json file",
    )
    parser.add_argument(
        "-p",
        "--period",
        type=positive_int,
        metavar="NUM",
        default=1,
        help="Specify the update frequency (unit: times/day), default value is 1",
    )
    parser.add_argument(
        "--shortcuts",
        nargs="*",
        type=str,
        metavar="URL",
        help="only shortcuts can be work",
    )
    parser.add_argument("--file", nargs="?", help=argparse.SUPPRESS)  # 仅运行在ipynb中
    parser.add_argument("--httpfs", action="store_true", help=argparse.SUPPRESS)
    # 解析参数
    args = parser.parse_args()
    parse.time_delay = args.delay
    parse.config = args.config
    parse.period = args.period
    parse.file = args.file
    parse.httpfs = args.httpfs
    # 检查并处理参数的状态
    if args.times is not None:
        parse.update_num = int(args.times[0])
    if args.shortcuts is not None:
        parse.update_num = 1
        parse.argument = "a-shell"
        parse.shortcuts_url_original = args.shortcuts
    if args.file is not None and ".json" in args.file:
        parse.update_num = 1
        parse.argument = ""
        parse.shortcuts_url_original = []
